@@ -36,13 +36,12 @@ import (
 	"k8s.io/component-base/logs"
 	"k8s.io/klog/v2/ktesting"
 	v1 "k8s.io/kube-scheduler/config/v1"
-	"k8s.io/kube-scheduler/config/v1beta3"
 	kubeschedulerconfig "k8s.io/kubernetes/pkg/scheduler/apis/config"
 	"k8s.io/kubernetes/pkg/scheduler/apis/config/latest"
 	configtesting "k8s.io/kubernetes/pkg/scheduler/apis/config/testing"
 	"k8s.io/kubernetes/pkg/scheduler/apis/config/testing/defaults"
 	"k8s.io/kubernetes/pkg/scheduler/framework/plugins/names"
-	"k8s.io/utils/pointer"
+	"k8s.io/utils/ptr"
 )
 
 func TestSchedulerOptions(t *testing.T) {
@@ -121,17 +120,6 @@ leaderElection:
 		t.Fatal(err)
 	}
 
-	v1beta3VersionConfig := filepath.Join(tmpDir, "scheduler_v1beta3_api_version.yaml")
-	if err := os.WriteFile(v1beta3VersionConfig, []byte(fmt.Sprintf(`
-apiVersion: kubescheduler.config.k8s.io/v1beta3
-kind: KubeSchedulerConfiguration
-clientConnection:
-  kubeconfig: '%s'
-leaderElection:
-  leaderElect: true`, configKubeconfig)), os.FileMode(0600)); err != nil {
-		t.Fatal(err)
-	}
-
 	unknownVersionConfig := filepath.Join(tmpDir, "scheduler_invalid_wrong_api_version.yaml")
 	if err := os.WriteFile(unknownVersionConfig, []byte(fmt.Sprintf(`
 apiVersion: kubescheduler.config.k8s.io/unknown
@@ -235,37 +223,6 @@ profiles:
 		t.Fatal(err)
 	}
 
-	// v1beta3 plugin config
-	v1beta3PluginConfigFile := filepath.Join(tmpDir, "v1beta3_plugin.yaml")
-	if err := os.WriteFile(v1beta3PluginConfigFile, []byte(fmt.Sprintf(`
-apiVersion: kubescheduler.config.k8s.io/v1beta3
-kind: KubeSchedulerConfiguration
-clientConnection:
-  kubeconfig: '%s'
-profiles:
-- plugins:
-    reserve:
-      enabled:
-      - name: foo
-      - name: bar
-      disabled:
-      - name: VolumeBinding
-    preBind:
-      enabled:
-      - name: foo
-      disabled:
-      - name: VolumeBinding
-  pluginConfig:
-  - name: InterPodAffinity
-    args:
-      hardPodAffinityWeight: 2
-  - name: foo
-    args:
-      bar: baz
-`, configKubeconfig)), os.FileMode(0600)); err != nil {
-		t.Fatal(err)
-	}
-
 	// multiple profiles config
 	multiProfilesConfig := filepath.Join(tmpDir, "multi-profiles.yaml")
 	if err := os.WriteFile(multiProfilesConfig, []byte(fmt.Sprintf(`
@@ -293,33 +250,6 @@ profiles:
 		t.Fatal(err)
 	}
 
-	// v1beta3 multiple profiles config
-	v1beta3MultiProfilesConfig := filepath.Join(tmpDir, "v1beta3_multi-profiles.yaml")
-	if err := os.WriteFile(v1beta3MultiProfilesConfig, []byte(fmt.Sprintf(`
-apiVersion: kubescheduler.config.k8s.io/v1beta3
-kind: KubeSchedulerConfiguration
-clientConnection:
-  kubeconfig: '%s'
-profiles:
-- schedulerName: "foo-profile"
-  plugins:
-    reserve:
-      enabled:
-      - name: foo
-      - name: VolumeBinding
-      disabled:
-      - name: VolumeBinding
-- schedulerName: "bar-profile"
-  plugins:
-    preBind:
-      disabled:
-      - name: VolumeBinding
-  pluginConfig:
-  - name: foo
-`, configKubeconfig)), os.FileMode(0600)); err != nil {
-		t.Fatal(err)
-	}
-
 	// high throughput profile config
 	highThroughputProfileConfig := filepath.Join(tmpDir, "high-throughput.yaml")
 	if err := os.WriteFile(highThroughputProfileConfig, []byte(fmt.Sprintf(`
@@ -349,7 +279,7 @@ profiles:
 
 	defaultPodInitialBackoffSeconds := int64(1)
 	defaultPodMaxBackoffSeconds := int64(10)
-	defaultPercentageOfNodesToScore := pointer.Int32(0)
+	defaultPercentageOfNodesToScore := ptr.To[int32](0)
 
 	testcases := []struct {
 		name             string
@@ -431,78 +361,6 @@ profiles:
 				},
 			},
 		},
-		{
-			name: "v1beta3 config file",
-			options: &Options{
-				ConfigFile: v1beta3VersionConfig,
-				ComponentConfig: func() *kubeschedulerconfig.KubeSchedulerConfiguration {
-					cfg := configtesting.V1beta3ToInternalWithDefaults(t, v1beta3.KubeSchedulerConfiguration{})
-					return cfg
-				}(),
-				SecureServing: (&apiserveroptions.SecureServingOptions{
-					ServerCert: apiserveroptions.GeneratableKeyCert{
-						CertDirectory: "/a/b/c",
-						PairName:      "kube-scheduler",
-					},
-					HTTP2MaxStreamsPerConnection: 47,
-				}).WithLoopback(),
-				Authentication: &apiserveroptions.DelegatingAuthenticationOptions{
-					CacheTTL:   10 * time.Second,
-					ClientCert: apiserveroptions.ClientCertAuthenticationOptions{},
-					RequestHeader: apiserveroptions.RequestHeaderAuthenticationOptions{
-						UsernameHeaders:     []string{"x-remote-user"},
-						GroupHeaders:        []string{"x-remote-group"},
-						ExtraHeaderPrefixes: []string{"x-remote-extra-"},
-					},
-					RemoteKubeConfigFileOptional: true,
-				},
-				Authorization: &apiserveroptions.DelegatingAuthorizationOptions{
-					AllowCacheTTL:                10 * time.Second,
-					DenyCacheTTL:                 10 * time.Second,
-					RemoteKubeConfigFileOptional: true,
-					AlwaysAllowPaths:             []string{"/healthz", "/readyz", "/livez"}, // note: this does not match /healthz/ or /healthz/*
-					AlwaysAllowGroups:            []string{"system:masters"},
-				},
-				Logs: logs.NewOptions(),
-			},
-			expectedUsername: "config",
-			expectedConfig: kubeschedulerconfig.KubeSchedulerConfiguration{
-				TypeMeta: metav1.TypeMeta{
-					APIVersion: v1beta3.SchemeGroupVersion.String(),
-				},
-				Parallelism:           16,
-				DelayCacheUntilActive: false,
-				DebuggingConfiguration: componentbaseconfig.DebuggingConfiguration{
-					EnableProfiling:           true,
-					EnableContentionProfiling: true,
-				},
-				LeaderElection: componentbaseconfig.LeaderElectionConfiguration{
-					LeaderElect:       true,
-					LeaseDuration:     metav1.Duration{Duration: 15 * time.Second},
-					RenewDeadline:     metav1.Duration{Duration: 10 * time.Second},
-					RetryPeriod:       metav1.Duration{Duration: 2 * time.Second},
-					ResourceLock:      "leases",
-					ResourceNamespace: "kube-system",
-					ResourceName:      "kube-scheduler",
-				},
-				ClientConnection: componentbaseconfig.ClientConnectionConfiguration{
-					Kubeconfig:  configKubeconfig,
-					QPS:         50,
-					Burst:       100,
-					ContentType: "application/vnd.kubernetes.protobuf",
-				},
-				PercentageOfNodesToScore: defaultPercentageOfNodesToScore,
-				PodInitialBackoffSeconds: defaultPodInitialBackoffSeconds,
-				PodMaxBackoffSeconds:     defaultPodMaxBackoffSeconds,
-				Profiles: []kubeschedulerconfig.KubeSchedulerProfile{
-					{
-						SchedulerName: "default-scheduler",
-						Plugins:       defaults.PluginsV1beta3,
-						PluginConfig:  defaults.PluginConfigsV1beta3,
-					},
-				},
-			},
-		},
 		{
 			name: "config file in componentconfig/v1alpha1",
 			options: &Options{
@@ -798,129 +656,6 @@ profiles:
 				},
 			},
 		},
-		{
-			name: "v1beta3 plugin config",
-			options: &Options{
-				ConfigFile: v1beta3PluginConfigFile,
-				Logs:       logs.NewOptions(),
-			},
-			expectedUsername: "config",
-			expectedConfig: kubeschedulerconfig.KubeSchedulerConfiguration{
-				TypeMeta: metav1.TypeMeta{
-					APIVersion: v1beta3.SchemeGroupVersion.String(),
-				},
-				Parallelism:           16,
-				DelayCacheUntilActive: false,
-				DebuggingConfiguration: componentbaseconfig.DebuggingConfiguration{
-					EnableProfiling:           true,
-					EnableContentionProfiling: true,
-				},
-				LeaderElection: componentbaseconfig.LeaderElectionConfiguration{
-					LeaderElect:       true,
-					LeaseDuration:     metav1.Duration{Duration: 15 * time.Second},
-					RenewDeadline:     metav1.Duration{Duration: 10 * time.Second},
-					RetryPeriod:       metav1.Duration{Duration: 2 * time.Second},
-					ResourceLock:      "leases",
-					ResourceNamespace: "kube-system",
-					ResourceName:      "kube-scheduler",
-				},
-				ClientConnection: componentbaseconfig.ClientConnectionConfiguration{
-					Kubeconfig:  configKubeconfig,
-					QPS:         50,
-					Burst:       100,
-					ContentType: "application/vnd.kubernetes.protobuf",
-				},
-				PercentageOfNodesToScore: defaultPercentageOfNodesToScore,
-				PodInitialBackoffSeconds: defaultPodInitialBackoffSeconds,
-				PodMaxBackoffSeconds:     defaultPodMaxBackoffSeconds,
-				Profiles: []kubeschedulerconfig.KubeSchedulerProfile{
-					{
-						SchedulerName: "default-scheduler",
-						Plugins: &kubeschedulerconfig.Plugins{
-							PreEnqueue: defaults.PluginsV1beta3.PreEnqueue,
-							QueueSort:  defaults.PluginsV1beta3.QueueSort,
-							PreFilter:  defaults.PluginsV1beta3.PreFilter,
-							Filter:     defaults.PluginsV1beta3.Filter,
-							PostFilter: defaults.PluginsV1beta3.PostFilter,
-							PreScore:   defaults.PluginsV1beta3.PreScore,
-							Score:      defaults.PluginsV1beta3.Score,
-							Reserve: kubeschedulerconfig.PluginSet{
-								Enabled: []kubeschedulerconfig.Plugin{
-									{Name: "foo"},
-									{Name: "bar"},
-								},
-								Disabled: []kubeschedulerconfig.Plugin{
-									{Name: "VolumeBinding"},
-								},
-							},
-							PreBind: kubeschedulerconfig.PluginSet{
-								Enabled: []kubeschedulerconfig.Plugin{
-									{Name: "foo"},
-								},
-								Disabled: []kubeschedulerconfig.Plugin{
-									{Name: "VolumeBinding"},
-								},
-							},
-							Bind:       defaults.PluginsV1beta3.Bind,
-							MultiPoint: defaults.PluginsV1beta3.MultiPoint,
-						},
-						PluginConfig: []kubeschedulerconfig.PluginConfig{
-							{
-								Name: "InterPodAffinity",
-								Args: &kubeschedulerconfig.InterPodAffinityArgs{
-									HardPodAffinityWeight: 2,
-								},
-							},
-							{
-								Name: "foo",
-								Args: &runtime.Unknown{
-									Raw:         []byte(`{"bar":"baz"}`),
-									ContentType: "application/json",
-								},
-							},
-							{
-								Name: "DefaultPreemption",
-								Args: &kubeschedulerconfig.DefaultPreemptionArgs{
-									MinCandidateNodesPercentage: 10,
-									MinCandidateNodesAbsolute:   100,
-								},
-							},
-							{
-								Name: "NodeAffinity",
-								Args: &kubeschedulerconfig.NodeAffinityArgs{},
-							},
-							{
-								Name: "NodeResourcesBalancedAllocation",
-								Args: &kubeschedulerconfig.NodeResourcesBalancedAllocationArgs{
-									Resources: []kubeschedulerconfig.ResourceSpec{{Name: "cpu", Weight: 1}, {Name: "memory", Weight: 1}},
-								},
-							},
-							{
-								Name: "NodeResourcesFit",
-								Args: &kubeschedulerconfig.NodeResourcesFitArgs{
-									ScoringStrategy: &kubeschedulerconfig.ScoringStrategy{
-										Type:      kubeschedulerconfig.LeastAllocated,
-										Resources: []kubeschedulerconfig.ResourceSpec{{Name: "cpu", Weight: 1}, {Name: "memory", Weight: 1}},
-									},
-								},
-							},
-							{
-								Name: "PodTopologySpread",
-								Args: &kubeschedulerconfig.PodTopologySpreadArgs{
-									DefaultingType: kubeschedulerconfig.SystemDefaulting,
-								},
-							},
-							{
-								Name: "VolumeBinding",
-								Args: &kubeschedulerconfig.VolumeBindingArgs{
-									BindTimeoutSeconds: 600,
-								},
-							},
-						},
-					},
-				},
-			},
-		},
 		{
 			name: "multiple profiles",
 			options: &Options{
@@ -1036,121 +771,6 @@ profiles:
 				},
 			},
 		},
-		{
-			name: "v1beta3 multiple profiles",
-			options: &Options{
-				ConfigFile: v1beta3MultiProfilesConfig,
-				Logs:       logs.NewOptions(),
-			},
-			expectedUsername: "config",
-			expectedConfig: kubeschedulerconfig.KubeSchedulerConfiguration{
-				TypeMeta: metav1.TypeMeta{
-					APIVersion: v1beta3.SchemeGroupVersion.String(),
-				},
-				Parallelism:           16,
-				DelayCacheUntilActive: false,
-				DebuggingConfiguration: componentbaseconfig.DebuggingConfiguration{
-					EnableProfiling:           true,
-					EnableContentionProfiling: true,
-				},
-				LeaderElection: componentbaseconfig.LeaderElectionConfiguration{
-					LeaderElect:       true,
-					LeaseDuration:     metav1.Duration{Duration: 15 * time.Second},
-					RenewDeadline:     metav1.Duration{Duration: 10 * time.Second},
-					RetryPeriod:       metav1.Duration{Duration: 2 * time.Second},
-					ResourceLock:      "leases",
-					ResourceNamespace: "kube-system",
-					ResourceName:      "kube-scheduler",
-				},
-				ClientConnection: componentbaseconfig.ClientConnectionConfiguration{
-					Kubeconfig:  configKubeconfig,
-					QPS:         50,
-					Burst:       100,
-					ContentType: "application/vnd.kubernetes.protobuf",
-				},
-				PercentageOfNodesToScore: defaultPercentageOfNodesToScore,
-				PodInitialBackoffSeconds: defaultPodInitialBackoffSeconds,
-				PodMaxBackoffSeconds:     defaultPodMaxBackoffSeconds,
-				Profiles: []kubeschedulerconfig.KubeSchedulerProfile{
-					{
-						SchedulerName: "foo-profile",
-						Plugins: &kubeschedulerconfig.Plugins{
-							MultiPoint: defaults.PluginsV1beta3.MultiPoint,
-							Reserve: kubeschedulerconfig.PluginSet{
-								Enabled: []kubeschedulerconfig.Plugin{
-									{Name: "foo"},
-									{Name: names.VolumeBinding},
-								},
-								Disabled: []kubeschedulerconfig.Plugin{
-									{Name: names.VolumeBinding},
-								},
-							},
-						},
-						PluginConfig: defaults.PluginConfigsV1beta3,
-					},
-					{
-						SchedulerName: "bar-profile",
-						Plugins: &kubeschedulerconfig.Plugins{
-							MultiPoint: defaults.PluginsV1beta3.MultiPoint,
-							PreBind: kubeschedulerconfig.PluginSet{
-								Disabled: []kubeschedulerconfig.Plugin{
-									{Name: names.VolumeBinding},
-								},
-							},
-						},
-						PluginConfig: []kubeschedulerconfig.PluginConfig{
-							{
-								Name: "foo",
-							},
-							{
-								Name: "DefaultPreemption",
-								Args: &kubeschedulerconfig.DefaultPreemptionArgs{
-									MinCandidateNodesPercentage: 10,
-									MinCandidateNodesAbsolute:   100,
-								},
-							},
-							{
-								Name: "InterPodAffinity",
-								Args: &kubeschedulerconfig.InterPodAffinityArgs{
-									HardPodAffinityWeight: 1,
-								},
-							},
-							{
-								Name: "NodeAffinity",
-								Args: &kubeschedulerconfig.NodeAffinityArgs{},
-							},
-							{
-								Name: "NodeResourcesBalancedAllocation",
-								Args: &kubeschedulerconfig.NodeResourcesBalancedAllocationArgs{
-									Resources: []kubeschedulerconfig.ResourceSpec{{Name: "cpu", Weight: 1}, {Name: "memory", Weight: 1}},
-								},
-							},
-							{
-								Name: "NodeResourcesFit",
-								Args: &kubeschedulerconfig.NodeResourcesFitArgs{
-									ScoringStrategy: &kubeschedulerconfig.ScoringStrategy{
-										Type:      kubeschedulerconfig.LeastAllocated,
-										Resources: []kubeschedulerconfig.ResourceSpec{{Name: "cpu", Weight: 1}, {Name: "memory", Weight: 1}},
-									},
-								},
-							},
-							{
-								Name: "PodTopologySpread",
-								Args: &kubeschedulerconfig.PodTopologySpreadArgs{
-									DefaultingType: kubeschedulerconfig.SystemDefaulting,
-								},
-							},
-							{
-								Name: "VolumeBinding",
-								Args: &kubeschedulerconfig.VolumeBindingArgs{
-									BindTimeoutSeconds: 600,
-								},
-							},
-						},
-					},
-				},
-			},
-		},
 		{
 			name: "no config",
 			options: &Options{
