@@ -18,10 +18,9 @@ package cmd
 
 import (
 	"bytes"
-	"context"
 	"fmt"
 	"io"
-	"io/ioutil"
+	"os"
 	"sort"
 	"strings"
 
@@ -29,44 +28,31 @@ import (
 	"github.com/pkg/errors"
 	"github.com/spf13/cobra"
 	flag "github.com/spf13/pflag"
-	"k8s.io/klog"
 
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/runtime"
 	"k8s.io/cli-runtime/pkg/genericclioptions"
-	clientset "k8s.io/client-go/kubernetes"
+	utilsexec "k8s.io/utils/exec"
+
 	kubeadmapi "k8s.io/kubernetes/cmd/kubeadm/app/apis/kubeadm"
 	kubeadmscheme "k8s.io/kubernetes/cmd/kubeadm/app/apis/kubeadm/scheme"
-	kubeadmapiv1beta2 "k8s.io/kubernetes/cmd/kubeadm/app/apis/kubeadm/v1beta2"
+	kubeadmapiv1old "k8s.io/kubernetes/cmd/kubeadm/app/apis/kubeadm/v1beta3"
+	kubeadmapiv1 "k8s.io/kubernetes/cmd/kubeadm/app/apis/kubeadm/v1beta4"
 	outputapischeme "k8s.io/kubernetes/cmd/kubeadm/app/apis/output/scheme"
-	outputapiv1alpha1 "k8s.io/kubernetes/cmd/kubeadm/app/apis/output/v1alpha1"
+	outputapiv1alpha2 "k8s.io/kubernetes/cmd/kubeadm/app/apis/output/v1alpha2"
 	"k8s.io/kubernetes/cmd/kubeadm/app/cmd/options"
-	phaseutil "k8s.io/kubernetes/cmd/kubeadm/app/cmd/phases"
 	cmdutil "k8s.io/kubernetes/cmd/kubeadm/app/cmd/util"
 	"k8s.io/kubernetes/cmd/kubeadm/app/componentconfigs"
 	"k8s.io/kubernetes/cmd/kubeadm/app/constants"
 	"k8s.io/kubernetes/cmd/kubeadm/app/features"
 	"k8s.io/kubernetes/cmd/kubeadm/app/images"
-	"k8s.io/kubernetes/cmd/kubeadm/app/phases/uploadconfig"
 	configutil "k8s.io/kubernetes/cmd/kubeadm/app/util/config"
-	kubeconfigutil "k8s.io/kubernetes/cmd/kubeadm/app/util/kubeconfig"
 	"k8s.io/kubernetes/cmd/kubeadm/app/util/output"
 	utilruntime "k8s.io/kubernetes/cmd/kubeadm/app/util/runtime"
-	utilsexec "k8s.io/utils/exec"
 )
 
-var (
-	// placeholderToken is only set statically to make kubeadm not randomize the token on every run
-	placeholderToken = kubeadmapiv1beta2.BootstrapToken{
-		Token: &kubeadmapiv1beta2.BootstrapTokenString{
-			ID:     "abcdef",
-			Secret: "0123456789abcdef",
-		},
-	}
-)
-
-// NewCmdConfig returns cobra.Command for "kubeadm config" command
-func NewCmdConfig(out io.Writer) *cobra.Command {
+// newCmdConfig returns cobra.Command for "kubeadm config" command
+func newCmdConfig(out io.Writer) *cobra.Command {
 	var kubeConfigFile string
 
 	cmd := &cobra.Command{
@@ -83,45 +69,50 @@ func NewCmdConfig(out io.Writer) *cobra.Command {
 		// cobra will print usage information, but still exit cleanly.
 		// We want to return an error code in these cases so that the
 		// user knows that their command was invalid.
-		RunE: cmdutil.SubCmdRunE("config"),
+		Run: cmdutil.SubCmdRun(),
 	}
 
 	options.AddKubeConfigFlag(cmd.PersistentFlags(), &kubeConfigFile)
 
 	kubeConfigFile = cmdutil.GetKubeConfigPath(kubeConfigFile)
-	cmd.AddCommand(NewCmdConfigPrint(out))
-	cmd.AddCommand(NewCmdConfigMigrate(out))
-	cmd.AddCommand(NewCmdConfigUpload(out, &kubeConfigFile))
-	cmd.AddCommand(NewCmdConfigView(out, &kubeConfigFile))
-	cmd.AddCommand(NewCmdConfigImages(out))
+	cmd.AddCommand(newCmdConfigPrint(out))
+	cmd.AddCommand(newCmdConfigMigrate(out))
+	cmd.AddCommand(newCmdConfigValidate(out))
+	cmd.AddCommand(newCmdConfigImages(out))
 	return cmd
 }
 
-// NewCmdConfigPrint returns cobra.Command for "kubeadm config print" command
-func NewCmdConfigPrint(out io.Writer) *cobra.Command {
+// newCmdConfigPrint returns cobra.Command for "kubeadm config print" command
+func newCmdConfigPrint(out io.Writer) *cobra.Command {
 	cmd := &cobra.Command{
 		Use:   "print",
 		Short: "Print configuration",
 		Long: dedent.Dedent(`
 			This command prints configurations for subcommands provided.
-			For details, see: https://godoc.org/k8s.io/kubernetes/cmd/kubeadm/app/apis/kubeadm/v1beta2`),
-		RunE: cmdutil.SubCmdRunE("print"),
+			For details, see: https://pkg.go.dev/k8s.io/kubernetes/cmd/kubeadm/app/apis/kubeadm#section-directories`),
+		Run: cmdutil.SubCmdRun(),
 	}
-	cmd.AddCommand(NewCmdConfigPrintInitDefaults(out))
-	cmd.AddCommand(NewCmdConfigPrintJoinDefaults(out))
+	cmd.AddCommand(newCmdConfigPrintInitDefaults(out))
+	cmd.AddCommand(newCmdConfigPrintJoinDefaults(out))
+	cmd.AddCommand(newCmdConfigPrintResetDefaults(out))
 	return cmd
 }
 
-// NewCmdConfigPrintInitDefaults returns cobra.Command for "kubeadm config print init-defaults" command
-func NewCmdConfigPrintInitDefaults(out io.Writer) *cobra.Command {
+// newCmdConfigPrintInitDefaults returns cobra.Command for "kubeadm config print init-defaults" command
+func newCmdConfigPrintInitDefaults(out io.Writer) *cobra.Command {
 	return newCmdConfigPrintActionDefaults(out, "init", getDefaultInitConfigBytes)
 }
 
-// NewCmdConfigPrintJoinDefaults returns cobra.Command for "kubeadm config print join-defaults" command
-func NewCmdConfigPrintJoinDefaults(out io.Writer) *cobra.Command {
+// newCmdConfigPrintJoinDefaults returns cobra.Command for "kubeadm config print join-defaults" command
+func newCmdConfigPrintJoinDefaults(out io.Writer) *cobra.Command {
 	return newCmdConfigPrintActionDefaults(out, "join", getDefaultNodeConfigBytes)
 }
 
+// newCmdConfigPrintResetDefaults returns cobra.Command for "kubeadm config print reset-defaults" command
+func newCmdConfigPrintResetDefaults(out io.Writer) *cobra.Command {
+	return newCmdConfigPrintActionDefaults(out, "reset", getDefaultResetConfigBytes)
+}
+
 func newCmdConfigPrintActionDefaults(out io.Writer, action string, configBytesProc func() ([]byte, error)) *cobra.Command {
 	kinds := []string{}
 	cmd := &cobra.Command{
@@ -132,7 +123,7 @@ func newCmdConfigPrintActionDefaults(out io.Writer, action string, configBytesPr
 
 			Note that sensitive values like the Bootstrap Token fields are replaced with placeholder values like %q in order to pass validation but
 			not perform the real computation for creating a token.
-		`), action, action, placeholderToken),
+		`), action, action, configutil.PlaceholderToken.Token.String()),
 		RunE: func(cmd *cobra.Command, args []string) error {
 			groups, err := mapLegacyKindsToGroups(kinds)
 			if err != nil {
@@ -140,9 +131,12 @@ func newCmdConfigPrintActionDefaults(out io.Writer, action string, configBytesPr
 			}
 			return runConfigPrintActionDefaults(out, groups, configBytesProc)
 		},
+		Args: cobra.NoArgs,
+	}
+	if action == "init" {
+		cmd.Flags().StringSliceVar(&kinds, "component-configs", kinds,
+			fmt.Sprintf("A comma-separated list for component config API objects to print the default values for. Available values: %v. If this flag is not set, no component configs will be printed.", getSupportedComponentConfigKinds()))
 	}
-	cmd.Flags().StringSliceVar(&kinds, "component-configs", kinds,
-		fmt.Sprintf("A comma-separated list for component config API objects to print the default values for. Available values: %v. If this flag is not set, no component configs will be printed.", getSupportedComponentConfigKinds()))
 	return cmd
 }
 
@@ -166,7 +160,7 @@ func runConfigPrintActionDefaults(out io.Writer, componentConfigs []string, conf
 }
 
 func getDefaultComponentConfigBytes(group string) ([]byte, error) {
-	defaultedInitConfig, err := getDefaultedInitConfig()
+	defaultedInitConfig, err := configutil.DefaultedStaticInitConfiguration()
 	if err != nil {
 		return []byte{}, err
 	}
@@ -189,7 +183,7 @@ var legacyKindToGroupMap = map[string]string{
 func getSupportedComponentConfigKinds() []string {
 	objects := []string{}
 	for componentType := range legacyKindToGroupMap {
-		objects = append(objects, string(componentType))
+		objects = append(objects, componentType)
 	}
 	sort.Strings(objects)
 	return objects
@@ -208,52 +202,52 @@ func mapLegacyKindsToGroups(kinds []string) ([]string, error) {
 	return groups, nil
 }
 
-func getDefaultedInitConfig() (*kubeadmapi.InitConfiguration, error) {
-	initCfg := &kubeadmapiv1beta2.InitConfiguration{
-		LocalAPIEndpoint: kubeadmapiv1beta2.APIEndpoint{AdvertiseAddress: "1.2.3.4"},
-		BootstrapTokens:  []kubeadmapiv1beta2.BootstrapToken{placeholderToken},
-		NodeRegistration: kubeadmapiv1beta2.NodeRegistrationOptions{
-			CRISocket: constants.DefaultDockerCRISocket, // avoid CRI detection
-		},
-	}
-	clusterCfg := &kubeadmapiv1beta2.ClusterConfiguration{
-		KubernetesVersion: constants.CurrentKubernetesVersion.String(), // avoid going to the Internet for the current Kubernetes version
-	}
-	return configutil.DefaultedInitConfiguration(initCfg, clusterCfg)
-}
-
 func getDefaultInitConfigBytes() ([]byte, error) {
-	internalcfg, err := getDefaultedInitConfig()
+	internalcfg, err := configutil.DefaultedStaticInitConfiguration()
 	if err != nil {
 		return []byte{}, err
 	}
 
-	return configutil.MarshalKubeadmConfigObject(internalcfg)
+	return configutil.MarshalKubeadmConfigObject(internalcfg, kubeadmapiv1old.SchemeGroupVersion)
 }
 
 func getDefaultNodeConfigBytes() ([]byte, error) {
-	internalcfg, err := configutil.DefaultedJoinConfiguration(&kubeadmapiv1beta2.JoinConfiguration{
-		Discovery: kubeadmapiv1beta2.Discovery{
-			BootstrapToken: &kubeadmapiv1beta2.BootstrapTokenDiscovery{
-				Token:                    placeholderToken.Token.String(),
+	opts := configutil.LoadOrDefaultConfigurationOptions{
+		SkipCRIDetect: true,
+	}
+	internalcfg, err := configutil.DefaultedJoinConfiguration(&kubeadmapiv1old.JoinConfiguration{
+		Discovery: kubeadmapiv1old.Discovery{
+			BootstrapToken: &kubeadmapiv1old.BootstrapTokenDiscovery{
+				Token:                    configutil.PlaceholderToken.Token.String(),
 				APIServerEndpoint:        "kube-apiserver:6443",
 				UnsafeSkipCAVerification: true, // TODO: UnsafeSkipCAVerification: true needs to be set for validation to pass, but shouldn't be recommended as the default
 			},
 		},
-		NodeRegistration: kubeadmapiv1beta2.NodeRegistrationOptions{
-			CRISocket: constants.DefaultDockerCRISocket, // avoid CRI detection
-		},
-	})
+	}, opts)
 	if err != nil {
 		return []byte{}, err
 	}
 
-	return configutil.MarshalKubeadmConfigObject(internalcfg)
+	return configutil.MarshalKubeadmConfigObject(internalcfg, kubeadmapiv1old.SchemeGroupVersion)
 }
 
-// NewCmdConfigMigrate returns cobra.Command for "kubeadm config migrate" command
-func NewCmdConfigMigrate(out io.Writer) *cobra.Command {
+func getDefaultResetConfigBytes() ([]byte, error) {
+	opts := configutil.LoadOrDefaultConfigurationOptions{
+		SkipCRIDetect: true,
+	}
+	internalcfg, err := configutil.DefaultedResetConfiguration(&kubeadmapiv1.ResetConfiguration{}, opts)
+	if err != nil {
+		return []byte{}, err
+	}
+
+	return configutil.MarshalKubeadmConfigObject(internalcfg, kubeadmapiv1.SchemeGroupVersion)
+}
+
+// newCmdConfigMigrate returns cobra.Command for "kubeadm config migrate" command
+func newCmdConfigMigrate(out io.Writer) *cobra.Command {
 	var oldCfgPath, newCfgPath string
+	var allowExperimental bool
+
 	cmd := &cobra.Command{
 		Use:   "migrate",
 		Short: "Read an older version of the kubeadm configuration API types from a file, and output the similar config object for the newer version",
@@ -270,18 +264,18 @@ func NewCmdConfigMigrate(out io.Writer) *cobra.Command {
 
 			In other words, the output of this command is what kubeadm actually would read internally if you
 			submitted this file to "kubeadm init"
-		`), kubeadmapiv1beta2.SchemeGroupVersion, kubeadmapiv1beta2.SchemeGroupVersion),
+		`), kubeadmapiv1old.SchemeGroupVersion, kubeadmapiv1old.SchemeGroupVersion),
 		RunE: func(cmd *cobra.Command, args []string) error {
 			if len(oldCfgPath) == 0 {
 				return errors.New("the --old-config flag is mandatory")
 			}
 
-			oldCfgBytes, err := ioutil.ReadFile(oldCfgPath)
+			oldCfgBytes, err := os.ReadFile(oldCfgPath)
 			if err != nil {
 				return err
 			}
 
-			outputBytes, err := configutil.MigrateOldConfig(oldCfgBytes)
+			outputBytes, err := configutil.MigrateOldConfig(oldCfgBytes, allowExperimental)
 			if err != nil {
 				return err
 			}
@@ -289,182 +283,79 @@ func NewCmdConfigMigrate(out io.Writer) *cobra.Command {
 			if newCfgPath == "" {
 				fmt.Fprint(out, string(outputBytes))
 			} else {
-				if err := ioutil.WriteFile(newCfgPath, outputBytes, 0644); err != nil {
+				if err := os.WriteFile(newCfgPath, outputBytes, 0644); err != nil {
 					return errors.Wrapf(err, "failed to write the new configuration to the file %q", newCfgPath)
 				}
 			}
 			return nil
 		},
+		Args: cobra.NoArgs,
 	}
 	cmd.Flags().StringVar(&oldCfgPath, "old-config", "", "Path to the kubeadm config file that is using an old API version and should be converted. This flag is mandatory.")
 	cmd.Flags().StringVar(&newCfgPath, "new-config", "", "Path to the resulting equivalent kubeadm config file using the new API version. Optional, if not specified output will be sent to STDOUT.")
+	cmd.Flags().BoolVar(&allowExperimental, options.AllowExperimentalAPI, false, "Allow migration to experimental, unreleased APIs.")
 	return cmd
 }
 
-// NewCmdConfigUpload (Deprecated) returns cobra.Command for "kubeadm config upload" command
-// Deprecated: please see kubeadm init phase upload-config
-func NewCmdConfigUpload(out io.Writer, kubeConfigFile *string) *cobra.Command {
-	cmd := &cobra.Command{
-		Deprecated: "please see kubeadm init phase upload-config",
-		Use:        "upload",
-		Short:      "Upload configuration about the current state, so that 'kubeadm upgrade' can later know how to configure the upgraded cluster",
-		RunE:       cmdutil.SubCmdRunE("upload"),
-	}
-
-	cmd.AddCommand(NewCmdConfigUploadFromFile(out, kubeConfigFile))
-	cmd.AddCommand(NewCmdConfigUploadFromFlags(out, kubeConfigFile))
-	return cmd
-}
-
-// NewCmdConfigView returns cobra.Command for "kubeadm config view" command
-func NewCmdConfigView(out io.Writer, kubeConfigFile *string) *cobra.Command {
-	return &cobra.Command{
-		Use:   "view",
-		Short: "View the kubeadm configuration stored inside the cluster",
-		Long: fmt.Sprintf(dedent.Dedent(`
-			Using this command, you can view the ConfigMap in the cluster where the configuration for kubeadm is located.
-
-			The configuration is located in the %q namespace in the %q ConfigMap.
-		`), metav1.NamespaceSystem, constants.KubeadmConfigConfigMap),
-		RunE: func(cmd *cobra.Command, args []string) error {
-			klog.V(1).Infoln("[config] retrieving ClientSet from file")
-			client, err := kubeconfigutil.ClientSetFromFile(*kubeConfigFile)
-			if err != nil {
-				return err
-			}
-
-			return RunConfigView(out, client)
-		},
-	}
-}
-
-// NewCmdConfigUploadFromFile verifies given Kubernetes config file and returns cobra.Command for
-// "kubeadm config upload from-file" command
-// Deprecated: please see kubeadm init phase upload-config
-func NewCmdConfigUploadFromFile(out io.Writer, kubeConfigFile *string) *cobra.Command {
+// newCmdConfigValidate returns cobra.Command for the "kubeadm config validate" command
+func newCmdConfigValidate(out io.Writer) *cobra.Command {
 	var cfgPath string
-	cmd := &cobra.Command{
-		Deprecated: "please see kubeadm init phase upload-config",
-		Use:        "from-file",
-		Short:      "Upload a configuration file to the in-cluster ConfigMap for kubeadm configuration",
-		Long: fmt.Sprintf(dedent.Dedent(`
-			Using this command, you can upload configuration to the ConfigMap in the cluster using the same config file you gave to 'kubeadm init'.
-			If you initialized your cluster using a v1.7.x or lower kubeadm client and used the --config option, you need to run this command with the
-			same config file before upgrading to v1.8 using 'kubeadm upgrade'.
+	var allowExperimental bool
 
-			The configuration is located in the %q namespace in the %q ConfigMap.
-		`), metav1.NamespaceSystem, constants.KubeadmConfigConfigMap),
+	cmd := &cobra.Command{
+		Use:   "validate",
+		Short: "Read a file containing the kubeadm configuration API and report any validation problems",
+		Long: fmt.Sprintf(dedent.Dedent(`
+			This command lets you validate a kubeadm configuration API file and report any warnings and errors.
+			If there are no errors the exit status will be zero, otherwise it will be non-zero.
+			Any unmarshaling problems such as unknown API fields will trigger errors. Unknown API versions and
+			fields with invalid values will also trigger errors. Any other errors or warnings may be reported
+			depending on contents of the input file.
+
+			In this version of kubeadm, the following API versions are supported:
+			- %s
+		`), kubeadmapiv1old.SchemeGroupVersion),
 		RunE: func(cmd *cobra.Command, args []string) error {
 			if len(cfgPath) == 0 {
-				return errors.New("the --config flag is mandatory")
+				return errors.Errorf("the --%s flag is mandatory", options.CfgPath)
 			}
 
-			klog.V(1).Infoln("[config] retrieving ClientSet from file")
-			client, err := kubeconfigutil.ClientSetFromFile(*kubeConfigFile)
+			cfgBytes, err := os.ReadFile(cfgPath)
 			if err != nil {
 				return err
 			}
 
-			// Default both statically and dynamically, convert to internal API type, and validate everything
-			internalcfg, err := configutil.LoadInitConfigurationFromFile(cfgPath)
-			if err != nil {
+			if err := configutil.ValidateConfig(cfgBytes, allowExperimental); err != nil {
 				return err
 			}
+			fmt.Fprintln(out, "ok")
 
-			// Upload the configuration using the file
-			klog.V(1).Infof("[config] uploading configuration")
-			return uploadconfig.UploadConfiguration(internalcfg, client)
+			return nil
 		},
+		Args: cobra.NoArgs,
 	}
 	options.AddConfigFlag(cmd.Flags(), &cfgPath)
+	cmd.Flags().BoolVar(&allowExperimental, options.AllowExperimentalAPI, false, "Allow validation of experimental, unreleased APIs.")
 	return cmd
 }
 
-// NewCmdConfigUploadFromFlags returns cobra.Command for "kubeadm config upload from-flags" command
-// Deprecated: please see kubeadm init phase upload-config
-func NewCmdConfigUploadFromFlags(out io.Writer, kubeConfigFile *string) *cobra.Command {
-	initCfg := &kubeadmapiv1beta2.InitConfiguration{}
-	kubeadmscheme.Scheme.Default(initCfg)
-
-	clusterCfg := &kubeadmapiv1beta2.ClusterConfiguration{}
-	kubeadmscheme.Scheme.Default(clusterCfg)
-
-	var featureGatesString string
-
-	cmd := &cobra.Command{
-		Deprecated: "please see kubeadm init phase upload-config",
-		Use:        "from-flags",
-		Short:      "Create the in-cluster configuration file for the first time from using flags",
-		Long: fmt.Sprintf(dedent.Dedent(`
-			Using this command, you can upload configuration to the ConfigMap in the cluster using the same flags you gave to 'kubeadm init'.
-			If you initialized your cluster using a v1.7.x or lower kubeadm client and set certain flags, you need to run this command with the
-			same flags before upgrading to v1.8 using 'kubeadm upgrade'.
-
-			The configuration is located in the %q namespace in the %q ConfigMap.
-		`), metav1.NamespaceSystem, constants.KubeadmConfigConfigMap),
-		RunE: func(cmd *cobra.Command, args []string) error {
-			var err error
-			klog.V(1).Infoln("[config] creating new FeatureGates")
-			if clusterCfg.FeatureGates, err = features.NewFeatureGate(&features.InitFeatureGates, featureGatesString); err != nil {
-				return nil
-			}
-			klog.V(1).Infoln("[config] retrieving ClientSet from file")
-			client, err := kubeconfigutil.ClientSetFromFile(*kubeConfigFile)
-			if err != nil {
-				return err
-			}
-
-			// KubernetesVersion is not used, but we set it explicitly to avoid the lookup
-			// of the version from the internet when executing DefaultedInitConfiguration
-			phaseutil.SetKubernetesVersion(clusterCfg)
-
-			// Default both statically and dynamically, convert to internal API type, and validate everything
-			klog.V(1).Infoln("[config] converting to internal API type")
-			internalcfg, err := configutil.DefaultedInitConfiguration(initCfg, clusterCfg)
-			if err != nil {
-				return err
-			}
-
-			// Finally, upload the configuration
-			klog.V(1).Infof("[config] uploading configuration")
-			return uploadconfig.UploadConfiguration(internalcfg, client)
-		},
-	}
-	AddInitConfigFlags(cmd.PersistentFlags(), initCfg)
-	AddClusterConfigFlags(cmd.PersistentFlags(), clusterCfg, &featureGatesString)
-	return cmd
-}
-
-// RunConfigView gets the configuration persisted in the cluster
-func RunConfigView(out io.Writer, client clientset.Interface) error {
-
-	klog.V(1).Infoln("[config] getting the cluster configuration")
-	cfgConfigMap, err := client.CoreV1().ConfigMaps(metav1.NamespaceSystem).Get(context.TODO(), constants.KubeadmConfigConfigMap, metav1.GetOptions{})
-	if err != nil {
-		return err
-	}
-	// No need to append \n as that already exists in the ConfigMap
-	fmt.Fprintf(out, "%s", cfgConfigMap.Data[constants.ClusterConfigurationConfigMapKey])
-	return nil
-}
-
-// NewCmdConfigImages returns the "kubeadm config images" command
-func NewCmdConfigImages(out io.Writer) *cobra.Command {
+// newCmdConfigImages returns the "kubeadm config images" command
+func newCmdConfigImages(out io.Writer) *cobra.Command {
 	cmd := &cobra.Command{
 		Use:   "images",
 		Short: "Interact with container images used by kubeadm",
-		RunE:  cmdutil.SubCmdRunE("images"),
+		Run:   cmdutil.SubCmdRun(),
 	}
-	cmd.AddCommand(NewCmdConfigImagesList(out, nil))
-	cmd.AddCommand(NewCmdConfigImagesPull())
+	cmd.AddCommand(newCmdConfigImagesList(out, nil))
+	cmd.AddCommand(newCmdConfigImagesPull())
 	return cmd
 }
 
-// NewCmdConfigImagesPull returns the `kubeadm config images pull` command
-func NewCmdConfigImagesPull() *cobra.Command {
-	externalClusterCfg := &kubeadmapiv1beta2.ClusterConfiguration{}
+// newCmdConfigImagesPull returns the `kubeadm config images pull` command
+func newCmdConfigImagesPull() *cobra.Command {
+	externalClusterCfg := &kubeadmapiv1old.ClusterConfiguration{}
 	kubeadmscheme.Scheme.Default(externalClusterCfg)
-	externalInitCfg := &kubeadmapiv1beta2.InitConfiguration{}
+	externalInitCfg := &kubeadmapiv1old.InitConfiguration{}
 	kubeadmscheme.Scheme.Default(externalInitCfg)
 	var cfgPath, featureGatesString string
 	var err error
@@ -477,7 +368,7 @@ func NewCmdConfigImagesPull() *cobra.Command {
 			if err != nil {
 				return err
 			}
-			internalcfg, err := configutil.LoadOrDefaultInitConfiguration(cfgPath, externalInitCfg, externalClusterCfg)
+			internalcfg, err := configutil.LoadOrDefaultInitConfiguration(cfgPath, externalInitCfg, externalClusterCfg, configutil.LoadOrDefaultConfigurationOptions{})
 			if err != nil {
 				return err
 			}
@@ -487,6 +378,7 @@ func NewCmdConfigImagesPull() *cobra.Command {
 			}
 			return PullControlPlaneImages(containerRuntime, &internalcfg.ClusterConfiguration)
 		},
+		Args: cobra.NoArgs,
 	}
 	AddImagesCommonConfigFlags(cmd.PersistentFlags(), externalClusterCfg, &cfgPath, &featureGatesString)
 	cmdutil.AddCRISocketFlag(cmd.PersistentFlags(), &externalInitCfg.NodeRegistration.CRISocket)
@@ -494,20 +386,6 @@ func NewCmdConfigImagesPull() *cobra.Command {
 	return cmd
 }
 
-// ImagesPull is the struct used to hold information relating to image pulling
-type ImagesPull struct {
-	runtime utilruntime.ContainerRuntime
-	images  []string
-}
-
-// NewImagesPull initializes and returns the `kubeadm config images pull` command
-func NewImagesPull(runtime utilruntime.ContainerRuntime, images []string) *ImagesPull {
-	return &ImagesPull{
-		runtime: runtime,
-		images:  images,
-	}
-}
-
 // PullControlPlaneImages pulls all images that the ImagesPull knows about
 func PullControlPlaneImages(runtime utilruntime.ContainerRuntime, cfg *kubeadmapi.ClusterConfiguration) error {
 	images := images.GetControlPlaneImages(cfg)
@@ -520,9 +398,9 @@ func PullControlPlaneImages(runtime utilruntime.ContainerRuntime, cfg *kubeadmap
 	return nil
 }
 
-// NewCmdConfigImagesList returns the "kubeadm config images list" command
-func NewCmdConfigImagesList(out io.Writer, mockK8sVersion *string) *cobra.Command {
-	externalcfg := &kubeadmapiv1beta2.ClusterConfiguration{}
+// newCmdConfigImagesList returns the "kubeadm config images list" command
+func newCmdConfigImagesList(out io.Writer, mockK8sVersion *string) *cobra.Command {
+	externalcfg := &kubeadmapiv1old.ClusterConfiguration{}
 	kubeadmscheme.Scheme.Default(externalcfg)
 	var cfgPath, featureGatesString string
 	var err error
@@ -546,7 +424,7 @@ func NewCmdConfigImagesList(out io.Writer, mockK8sVersion *string) *cobra.Comman
 
 			printer, err := outputFlags.ToPrinter()
 			if err != nil {
-				return err
+				return errors.Wrap(err, "could not construct output printer")
 			}
 
 			imagesList, err := NewImagesList(cfgPath, externalcfg)
@@ -556,6 +434,7 @@ func NewCmdConfigImagesList(out io.Writer, mockK8sVersion *string) *cobra.Comman
 
 			return imagesList.Run(out, printer)
 		},
+		Args: cobra.NoArgs,
 	}
 	outputFlags.AddFlags(cmd)
 	AddImagesCommonConfigFlags(cmd.PersistentFlags(), externalcfg, &cfgPath, &featureGatesString)
@@ -563,15 +442,10 @@ func NewCmdConfigImagesList(out io.Writer, mockK8sVersion *string) *cobra.Comman
 }
 
 // NewImagesList returns the underlying struct for the "kubeadm config images list" command
-func NewImagesList(cfgPath string, cfg *kubeadmapiv1beta2.ClusterConfiguration) (*ImagesList, error) {
-	// Avoid running the CRI auto-detection code as we don't need it
-	versionedInitCfg := &kubeadmapiv1beta2.InitConfiguration{
-		NodeRegistration: kubeadmapiv1beta2.NodeRegistrationOptions{
-			CRISocket: constants.DefaultDockerCRISocket,
-		},
-	}
-
-	initcfg, err := configutil.LoadOrDefaultInitConfiguration(cfgPath, versionedInitCfg, cfg)
+func NewImagesList(cfgPath string, cfg *kubeadmapiv1old.ClusterConfiguration) (*ImagesList, error) {
+	initcfg, err := configutil.LoadOrDefaultInitConfiguration(cfgPath, &kubeadmapiv1old.InitConfiguration{}, cfg, configutil.LoadOrDefaultConfigurationOptions{
+		SkipCRIDetect: true,
+	})
 	if err != nil {
 		return nil, errors.Wrap(err, "could not convert cfg to an internal cfg")
 	}
@@ -594,7 +468,7 @@ type imageTextPrinter struct {
 // PrintObj is an implementation of ResourcePrinter.PrintObj for plain text output
 func (itp *imageTextPrinter) PrintObj(obj runtime.Object, writer io.Writer) error {
 	var err error
-	if imgs, ok := obj.(*outputapiv1alpha1.Images); ok {
+	if imgs, ok := obj.(*outputapiv1alpha2.Images); ok {
 		_, err = fmt.Fprintln(writer, strings.Join(imgs.Images, "\n"))
 	} else {
 		err = errors.New("unexpected object type")
@@ -605,7 +479,7 @@ func (itp *imageTextPrinter) PrintObj(obj runtime.Object, writer io.Writer) erro
 // imageTextPrintFlags provides flags necessary for printing image in a text form.
 type imageTextPrintFlags struct{}
 
-// ToPrinter returns kubeadm printer for the text output format
+// ToPrinter returns a kubeadm printer for the text output format
 func (ipf *imageTextPrintFlags) ToPrinter(outputFormat string) (output.Printer, error) {
 	if outputFormat == output.TextOutput {
 		return &imageTextPrinter{}, nil
@@ -617,7 +491,7 @@ func (ipf *imageTextPrintFlags) ToPrinter(outputFormat string) (output.Printer,
 func (i *ImagesList) Run(out io.Writer, printer output.Printer) error {
 	imgs := images.GetControlPlaneImages(&i.cfg.ClusterConfiguration)
 
-	if err := printer.PrintObj(&outputapiv1alpha1.Images{Images: imgs}, out); err != nil {
+	if err := printer.PrintObj(&outputapiv1alpha2.Images{Images: imgs}, out); err != nil {
 		return errors.Wrap(err, "unable to print images")
 	}
 
@@ -625,7 +499,7 @@ func (i *ImagesList) Run(out io.Writer, printer output.Printer) error {
 }
 
 // AddImagesCommonConfigFlags adds the flags that configure kubeadm (and affect the images kubeadm will use)
-func AddImagesCommonConfigFlags(flagSet *flag.FlagSet, cfg *kubeadmapiv1beta2.ClusterConfiguration, cfgPath *string, featureGatesString *string) {
+func AddImagesCommonConfigFlags(flagSet *flag.FlagSet, cfg *kubeadmapiv1old.ClusterConfiguration, cfgPath *string, featureGatesString *string) {
 	options.AddKubernetesVersionFlag(flagSet, &cfg.KubernetesVersion)
 	options.AddFeatureGatesStringFlag(flagSet, featureGatesString)
 	options.AddImageMetaFlags(flagSet, &cfg.ImageRepository)
