@@ -25,6 +25,7 @@ import (
 	"fmt"
 	"net"
 	"os"
+	"path/filepath"
 	"reflect"
 	"testing"
 
@@ -51,7 +52,7 @@ func TestMain(m *testing.M) {
 		Config: certutil.Config{
 			CommonName: "Root CA 1",
 		},
-		PublicKeyAlgorithm: x509.RSA,
+		EncryptionAlgorithm: kubeadmapi.EncryptionAlgorithmRSA,
 	})
 	if err != nil {
 		panic(fmt.Sprintf("Failed generating Root CA: %v", err))
@@ -111,7 +112,7 @@ func TestHasServerAuth(t *testing.T) {
 	// Override NewPrivateKey to reuse the same key for all certs
 	// since this test is only checking cert.ExtKeyUsage
 	privateKeyFunc := NewPrivateKey
-	NewPrivateKey = func(x509.PublicKeyAlgorithm) (crypto.Signer, error) {
+	NewPrivateKey = func(kubeadmapi.EncryptionAlgorithmType) (crypto.Signer, error) {
 		return rootCAKey, nil
 	}
 	defer func() {
@@ -140,7 +141,7 @@ func TestHasServerAuth(t *testing.T) {
 					CommonName: "test",
 					Usages:     []x509.ExtKeyUsage{x509.ExtKeyUsageServerAuth},
 				},
-				PublicKeyAlgorithm: x509.ECDSA,
+				EncryptionAlgorithm: kubeadmapi.EncryptionAlgorithmECDSA,
 			},
 			expected: true,
 		},
@@ -262,12 +263,16 @@ func TestCertOrKeyExist(t *testing.T) {
 	}
 	defer os.RemoveAll(tmpdir)
 
-	caCert := &x509.Certificate{}
-	actual := WriteCertAndKey(tmpdir, "foo", caCert, rootCAKey)
-	if actual != nil {
+	if err = WriteCertAndKey(tmpdir, "foo-0", rootCACert, rootCAKey); err != nil {
 		t.Errorf(
 			"failed WriteCertAndKey with an error: %v",
-			actual,
+			err,
+		)
+	}
+	if err = WriteCert(tmpdir, "foo-1", rootCACert); err != nil {
+		t.Errorf(
+			"failed WriteCert with an error: %v",
+			err,
 		)
 	}
 
@@ -284,9 +289,15 @@ func TestCertOrKeyExist(t *testing.T) {
 			expected: false,
 		},
 		{
-			desc:     "valid path and name",
+			desc:     "valid path and name, both cert and key exist",
 			path:     tmpdir,
-			name:     "foo",
+			name:     "foo-0",
+			expected: true,
+		},
+		{
+			desc:     "valid path and name, only cert exist",
+			path:     tmpdir,
+			name:     "foo-1",
 			expected: true,
 		},
 	}
@@ -528,38 +539,44 @@ func TestTryLoadKeyFromDisk(t *testing.T) {
 
 func TestPathsForCertAndKey(t *testing.T) {
 	crtPath, keyPath := PathsForCertAndKey("/foo", "bar")
-	if crtPath != "/foo/bar.crt" {
+	expectedPath := filepath.FromSlash("/foo/bar.crt")
+	if crtPath != expectedPath {
 		t.Errorf("unexpected certificate path: %s", crtPath)
 	}
-	if keyPath != "/foo/bar.key" {
+	expectedPath = filepath.FromSlash("/foo/bar.key")
+	if keyPath != expectedPath {
 		t.Errorf("unexpected key path: %s", keyPath)
 	}
 }
 
 func TestPathForCert(t *testing.T) {
 	crtPath := pathForCert("/foo", "bar")
-	if crtPath != "/foo/bar.crt" {
+	expectedPath := filepath.FromSlash("/foo/bar.crt")
+	if crtPath != expectedPath {
 		t.Errorf("unexpected certificate path: %s", crtPath)
 	}
 }
 
 func TestPathForKey(t *testing.T) {
 	keyPath := pathForKey("/foo", "bar")
-	if keyPath != "/foo/bar.key" {
+	expectedPath := filepath.FromSlash("/foo/bar.key")
+	if keyPath != expectedPath {
 		t.Errorf("unexpected certificate path: %s", keyPath)
 	}
 }
 
 func TestPathForPublicKey(t *testing.T) {
 	pubPath := pathForPublicKey("/foo", "bar")
-	if pubPath != "/foo/bar.pub" {
+	expectedPath := filepath.FromSlash("/foo/bar.pub")
+	if pubPath != expectedPath {
 		t.Errorf("unexpected certificate path: %s", pubPath)
 	}
 }
 
 func TestPathForCSR(t *testing.T) {
 	csrPath := pathForCSR("/foo", "bar")
-	if csrPath != "/foo/bar.csr" {
+	expectedPath := filepath.FromSlash("/foo/bar.csr")
+	if csrPath != expectedPath {
 		t.Errorf("unexpected certificate path: %s", csrPath)
 	}
 }
