@@ -18,6 +18,7 @@ KUBE_ROOT=$(dirname "${BASH_SOURCE[0]}")/../..
 source "${KUBE_ROOT}/hack/lib/init.sh"
 
 kube::golang::setup_env
+kube::golang::setup_gomaxprocs
 
 # start the cache mutation detector by default so that cache mutators will be found
 KUBE_CACHE_MUTATION_DETECTOR="${KUBE_CACHE_MUTATION_DETECTOR:-true}"
@@ -53,7 +54,7 @@ ssh_options=${SSH_OPTIONS:-}
 kubelet_config_file=${KUBELET_CONFIG_FILE:-"test/e2e_node/jenkins/default-kubelet-config.yaml"}
 
 # Parse the flags to pass to ginkgo
-ginkgoflags=""
+ginkgoflags="-timeout=24h"
 if [[ ${parallelism} -gt 1 ]]; then
   ginkgoflags="${ginkgoflags} -nodes=${parallelism} "
 fi
@@ -66,8 +67,8 @@ if [[ ${skip} != "" ]]; then
   ginkgoflags="${ginkgoflags} -skip=\"${skip}\" "
 fi
 
-if [[ ${run_until_failure} != "" ]]; then
-  ginkgoflags="${ginkgoflags} -untilItFails=${run_until_failure} "
+if [[ ${run_until_failure} == "true" ]]; then
+  ginkgoflags="${ginkgoflags} --until-it-fails=true "
 fi
 
 # Setup the directory to copy test artifacts (logs, junit.xml, etc) from remote host to local host
@@ -84,24 +85,25 @@ if [[ -n ${image_service_endpoint} ]] ; then
   test_args="--image-service-endpoint=${image_service_endpoint} ${test_args}"
 fi
 
+if [[ "${test_args}" != *"prepull-images"* ]]; then
+  test_args="--prepull-images=${PREPULL_IMAGES:-false}  ${test_args}"
+fi
 
 if [ "${remote}" = true ] && [ "${remote_mode}" = gce ] ; then
   # The following options are only valid in remote GCE run.
   images=${IMAGES:-""}
   hosts=${HOSTS:-""}
-  image_project=${IMAGE_PROJECT:-"kubernetes-node-e2e-images"}
+  image_project=${IMAGE_PROJECT:-"cos-cloud"}
   metadata=${INSTANCE_METADATA:-""}
-  list_images=${LIST_IMAGES:-false}
-  if  [[ ${list_images} == "true" ]]; then
-    gcloud compute images list --project="${image_project}" | grep "e2e-node"
-    exit 0
-  fi
   gubernator=${GUBERNATOR:-"false"}
+  instance_type=${INSTANCE_TYPE:-""}
+  node_env="${NODE_ENV:-""}"
   image_config_file=${IMAGE_CONFIG_FILE:-""}
   image_config_dir=${IMAGE_CONFIG_DIR:-""}
+  use_dockerized_build=${USE_DOCKERIZED_BUILD:-"false"}
+  target_build_arch=${TARGET_BUILD_ARCH:-""}
   runtime_config=${RUNTIME_CONFIG:-""}
   if [[ ${hosts} == "" && ${images} == "" && ${image_config_file} == "" ]]; then
-    image_project="${IMAGE_PROJECT:-"cos-cloud"}"
     gci_image=$(gcloud compute images list --project "${image_project}" \
     --no-standard-images --filter="name ~ 'cos-beta.*'" --format="table[no-heading](name)")
     images=${gci_image}
@@ -117,9 +119,9 @@ if [ "${remote}" = true ] && [ "${remote_mode}" = gce ] ; then
   fi
 
   # Get the compute zone
-  zone=${ZONE:-"$(gcloud info --format='value(config.properties.compute.zone)')"}
+  zone=${ZONE:-"$(gcloud info --format='value(config.properties.compute.zone.value)')"}
   if [[ ${zone} == "" ]]; then
-    echo "Could not find gcloud compute/zone when running: \`gcloud info --format='value(config.properties.compute.zone)'\`"
+    echo "Could not find gcloud compute/zone when running: \`gcloud info --format='value(config.properties.compute.zone.value)'\`"
     exit 1
   fi
 
@@ -160,15 +162,30 @@ if [ "${remote}" = true ] && [ "${remote_mode}" = gce ] ; then
   echo "Project: ${project}"
   echo "Image Project: ${image_project}"
   echo "Compute/Zone: ${zone}"
-  echo "Images: ${images}"
-  echo "Hosts: ${hosts}"
+  if [[ -n ${images} ]]; then
+    echo "Images: ${images}"
+  fi
+  if [[ -n ${hosts} ]]; then
+    echo "Hosts: ${hosts}"
+  fi
+  echo "Test Args: ${test_args}"
   echo "Ginkgo Flags: ${ginkgoflags}"
-  echo "Instance Metadata: ${metadata}"
-  echo "Image Config File: ${image_config_file}"
+  if [[ -n ${metadata} ]]; then
+    echo "Instance Metadata: ${metadata}"
+  fi
+  if [[ -n ${node_env} ]]; then
+    echo "Node-env: \"${node_env}\""
+  fi
+  if [[ -n ${image_config_file} ]]; then
+    echo "Image Config File: ${image_config_dir}/${image_config_file}"
+  fi
+  if [[ -n ${instance_type} ]]; then
+    echo "Instance Type: ${instance_type}"
+  fi
   echo "Kubelet Config File: ${kubelet_config_file}"
 
   # Invoke the runner
-  go run test/e2e_node/runner/remote/run_remote.go  --logtostderr --vmodule=*=4 --ssh-env="gce" \
+  go run test/e2e_node/runner/remote/run_remote.go  --vmodule=*=4 --ssh-env="gce" \
     --zone="${zone}" --project="${project}" --gubernator="${gubernator}" \
     --hosts="${hosts}" --images="${images}" --cleanup="${cleanup}" \
     --results-dir="${artifacts}" --ginkgo-flags="${ginkgoflags}" --runtime-config="${runtime_config}" \
@@ -177,7 +194,9 @@ if [ "${remote}" = true ] && [ "${remote_mode}" = gce ] ; then
     --image-config-file="${image_config_file}" --system-spec-name="${system_spec_name}" \
     --runtime-config="${runtime_config}" --preemptible-instances="${preemptible_instances}" \
     --ssh-user="${ssh_user}" --ssh-key="${ssh_key}" --ssh-options="${ssh_options}" \
-    --image-config-dir="${image_config_dir}" \
+    --image-config-dir="${image_config_dir}" --node-env="${node_env}" \
+    --use-dockerized-build="${use_dockerized_build}" --instance-type="${instance_type}" \
+    --target-build-arch="${target_build_arch}" \
     --extra-envs="${extra_envs}" --kubelet-config-file="${kubelet_config_file}"  --test-suite="${test_suite}" \
     "${timeout_arg}" \
     2>&1 | tee -i "${artifacts}/build-log.txt"
@@ -195,7 +214,7 @@ elif [ "${remote}" = true ] && [ "${remote_mode}" = ssh ] ; then
   test_args='--kubelet-flags="--cluster-domain='${KUBE_DNS_DOMAIN:-cluster.local}'" '${test_args}
 
   # Invoke the runner
-  go run test/e2e_node/runner/remote/run_remote.go  --mode="ssh" --logtostderr --vmodule=*=4 \
+  go run test/e2e_node/runner/remote/run_remote.go  --mode="ssh" --vmodule=*=4 \
     --hosts="${hosts}" --results-dir="${artifacts}" --ginkgo-flags="${ginkgoflags}" \
     --test_args="${test_args}" --system-spec-name="${system_spec_name}" \
     --runtime-config="${runtime_config}" \
@@ -209,6 +228,8 @@ else
   # Refresh sudo credentials if needed
   if ping -c 1 -q metadata.google.internal &> /dev/null; then
     echo 'Running on GCE, not asking for sudo credentials'
+  elif ping -c 1 -q 169.254.169.254 &> /dev/null; then
+    echo 'Running on AWS, not asking for sudo credentials'
   elif sudo --non-interactive "$(which bash)" -c true 2> /dev/null; then
     # if we can run bash without a password, it's a pretty safe bet that either
     # we can run any command without a password, or that sudo credentials
@@ -228,7 +249,7 @@ else
   go run test/e2e_node/runner/local/run_local.go \
     --system-spec-name="${system_spec_name}" --extra-envs="${extra_envs}" \
     --ginkgo-flags="${ginkgoflags}" \
-    --test-flags="--alsologtostderr --v 4 --report-dir=${artifacts} --node-name $(hostname) ${test_args}" \
+    --test-flags="--v 4 --report-dir=${artifacts} --node-name $(hostname) ${test_args}" \
     --runtime-config="${runtime_config}" \
     --kubelet-config-file="${kubelet_config_file}" \
     --build-dependencies=true 2>&1 | tee -i "${artifacts}/build-log.txt"
