@@ -34,7 +34,7 @@ import (
 	. "k8s.io/kubernetes/pkg/apis/apps/v1beta2"
 	_ "k8s.io/kubernetes/pkg/apis/core/install"
 	"k8s.io/kubernetes/pkg/features"
-	utilpointer "k8s.io/utils/pointer"
+	"k8s.io/utils/ptr"
 )
 
 func TestSetDefaultDaemonSetSpec(t *testing.T) {
@@ -86,7 +86,7 @@ func TestSetDefaultDaemonSetSpec(t *testing.T) {
 							MaxSurge:       &maxSurge,
 						},
 					},
-					RevisionHistoryLimit: utilpointer.Int32(10),
+					RevisionHistoryLimit: ptr.To[int32](10),
 				},
 			},
 		},
@@ -99,7 +99,7 @@ func TestSetDefaultDaemonSetSpec(t *testing.T) {
 				},
 				Spec: appsv1beta2.DaemonSetSpec{
 					Template:             defaultTemplate,
-					RevisionHistoryLimit: utilpointer.Int32(1),
+					RevisionHistoryLimit: ptr.To[int32](1),
 				},
 			},
 			expected: &appsv1beta2.DaemonSet{
@@ -117,7 +117,7 @@ func TestSetDefaultDaemonSetSpec(t *testing.T) {
 							MaxSurge:       &maxSurge,
 						},
 					},
-					RevisionHistoryLimit: utilpointer.Int32(1),
+					RevisionHistoryLimit: ptr.To[int32](1),
 				},
 			},
 		},
@@ -136,7 +136,7 @@ func TestSetDefaultDaemonSetSpec(t *testing.T) {
 					UpdateStrategy: appsv1beta2.DaemonSetUpdateStrategy{
 						Type: appsv1beta2.OnDeleteDaemonSetStrategyType,
 					},
-					RevisionHistoryLimit: utilpointer.Int32(10),
+					RevisionHistoryLimit: ptr.To[int32](10),
 				},
 			},
 		},
@@ -154,7 +154,7 @@ func TestSetDefaultDaemonSetSpec(t *testing.T) {
 							MaxSurge:       &maxSurge,
 						},
 					},
-					RevisionHistoryLimit: utilpointer.Int32(10),
+					RevisionHistoryLimit: ptr.To[int32](10),
 				},
 			},
 		},
@@ -175,15 +175,6 @@ func TestSetDefaultDaemonSetSpec(t *testing.T) {
 	}
 }
 
-func getMaxUnavailable(maxUnavailable int) *intstr.IntOrString {
-	maxUnavailableIntOrStr := intstr.FromInt(maxUnavailable)
-	return &maxUnavailableIntOrStr
-}
-
-func getPartition(partition int32) *int32 {
-	return &partition
-}
-
 func TestSetDefaultStatefulSet(t *testing.T) {
 	defaultLabels := map[string]string{"foo": "bar"}
 	var defaultPartition int32 = 0
@@ -233,7 +224,7 @@ func TestSetDefaultStatefulSet(t *testing.T) {
 							Partition: &defaultPartition,
 						},
 					},
-					RevisionHistoryLimit: utilpointer.Int32(10),
+					RevisionHistoryLimit: ptr.To[int32](10),
 				},
 			},
 		},
@@ -259,7 +250,7 @@ func TestSetDefaultStatefulSet(t *testing.T) {
 					UpdateStrategy: appsv1beta2.StatefulSetUpdateStrategy{
 						Type: appsv1beta2.OnDeleteStatefulSetStrategyType,
 					},
-					RevisionHistoryLimit: utilpointer.Int32(10),
+					RevisionHistoryLimit: ptr.To[int32](10),
 				},
 			},
 		},
@@ -286,7 +277,7 @@ func TestSetDefaultStatefulSet(t *testing.T) {
 							Partition: &defaultPartition,
 						},
 					},
-					RevisionHistoryLimit: utilpointer.Int32(10),
+					RevisionHistoryLimit: ptr.To[int32](10),
 				},
 			},
 		},
@@ -308,10 +299,10 @@ func TestSetDefaultStatefulSet(t *testing.T) {
 					UpdateStrategy: appsv1beta2.StatefulSetUpdateStrategy{
 						Type: appsv1beta2.RollingUpdateStatefulSetStrategyType,
 						RollingUpdate: &appsv1beta2.RollingUpdateStatefulSetStrategy{
-							Partition: getPartition(0),
+							Partition: ptr.To[int32](0),
 						},
 					},
-					RevisionHistoryLimit: utilpointer.Int32(10),
+					RevisionHistoryLimit: ptr.To[int32](10),
 				},
 			},
 			enableMaxUnavailablePolicy: false,
@@ -324,7 +315,7 @@ func TestSetDefaultStatefulSet(t *testing.T) {
 					UpdateStrategy: appsv1beta2.StatefulSetUpdateStrategy{
 						RollingUpdate: &appsv1beta2.RollingUpdateStatefulSetStrategy{
 							Partition:      &defaultPartition,
-							MaxUnavailable: getMaxUnavailable(1),
+							MaxUnavailable: ptr.To(intstr.FromInt32(1)),
 						},
 					},
 				},
@@ -340,11 +331,11 @@ func TestSetDefaultStatefulSet(t *testing.T) {
 					UpdateStrategy: appsv1beta2.StatefulSetUpdateStrategy{
 						Type: appsv1beta2.RollingUpdateStatefulSetStrategyType,
 						RollingUpdate: &appsv1beta2.RollingUpdateStatefulSetStrategy{
-							Partition:      getPartition(0),
-							MaxUnavailable: getMaxUnavailable(1),
+							Partition:      ptr.To[int32](0),
+							MaxUnavailable: ptr.To(intstr.FromInt32(1)),
 						},
 					},
-					RevisionHistoryLimit: utilpointer.Int32(10),
+					RevisionHistoryLimit: ptr.To[int32](10),
 				},
 			},
 			enableMaxUnavailablePolicy: false,
@@ -357,7 +348,7 @@ func TestSetDefaultStatefulSet(t *testing.T) {
 					UpdateStrategy: appsv1beta2.StatefulSetUpdateStrategy{
 						RollingUpdate: &appsv1beta2.RollingUpdateStatefulSetStrategy{
 							Partition:      &notTheDefaultPartition,
-							MaxUnavailable: getMaxUnavailable(3),
+							MaxUnavailable: ptr.To(intstr.FromInt32(3)),
 						},
 					},
 				},
@@ -373,11 +364,11 @@ func TestSetDefaultStatefulSet(t *testing.T) {
 					UpdateStrategy: appsv1beta2.StatefulSetUpdateStrategy{
 						Type: appsv1beta2.RollingUpdateStatefulSetStrategyType,
 						RollingUpdate: &appsv1beta2.RollingUpdateStatefulSetStrategy{
-							Partition:      getPartition(42),
-							MaxUnavailable: getMaxUnavailable(3),
+							Partition:      ptr.To[int32](42),
+							MaxUnavailable: ptr.To(intstr.FromInt32(3)),
 						},
 					},
-					RevisionHistoryLimit: utilpointer.Int32(10),
+					RevisionHistoryLimit: ptr.To[int32](10),
 				},
 			},
 			enableMaxUnavailablePolicy: false,
@@ -400,11 +391,11 @@ func TestSetDefaultStatefulSet(t *testing.T) {
 					UpdateStrategy: appsv1beta2.StatefulSetUpdateStrategy{
 						Type: appsv1beta2.RollingUpdateStatefulSetStrategyType,
 						RollingUpdate: &appsv1beta2.RollingUpdateStatefulSetStrategy{
-							Partition:      getPartition(0),
-							MaxUnavailable: getMaxUnavailable(1),
+							Partition:      ptr.To[int32](0),
+							MaxUnavailable: ptr.To(intstr.FromInt32(1)),
 						},
 					},
-					RevisionHistoryLimit: utilpointer.Int32(10),
+					RevisionHistoryLimit: ptr.To[int32](10),
 				},
 			},
 			enableMaxUnavailablePolicy: true,
@@ -417,7 +408,7 @@ func TestSetDefaultStatefulSet(t *testing.T) {
 					UpdateStrategy: appsv1beta2.StatefulSetUpdateStrategy{
 						RollingUpdate: &appsv1beta2.RollingUpdateStatefulSetStrategy{
 							Partition:      &notTheDefaultPartition,
-							MaxUnavailable: getMaxUnavailable(3),
+							MaxUnavailable: ptr.To(intstr.FromInt32(3)),
 						},
 					},
 				},
@@ -433,11 +424,11 @@ func TestSetDefaultStatefulSet(t *testing.T) {
 					UpdateStrategy: appsv1beta2.StatefulSetUpdateStrategy{
 						Type: appsv1beta2.RollingUpdateStatefulSetStrategyType,
 						RollingUpdate: &appsv1beta2.RollingUpdateStatefulSetStrategy{
-							Partition:      getPartition(42),
-							MaxUnavailable: getMaxUnavailable(3),
+							Partition:      ptr.To[int32](42),
+							MaxUnavailable: ptr.To(intstr.FromInt32(3)),
 						},
 					},
-					RevisionHistoryLimit: utilpointer.Int32(10),
+					RevisionHistoryLimit: ptr.To[int32](10),
 				},
 			},
 			enableMaxUnavailablePolicy: true,
@@ -463,7 +454,7 @@ func TestSetDefaultStatefulSet(t *testing.T) {
 							Partition: &defaultPartition,
 						},
 					},
-					RevisionHistoryLimit: utilpointer.Int32(10),
+					RevisionHistoryLimit: ptr.To[int32](10),
 					PersistentVolumeClaimRetentionPolicy: &appsv1beta2.StatefulSetPersistentVolumeClaimRetentionPolicy{
 						WhenDeleted: appsv1beta2.RetainPersistentVolumeClaimRetentionPolicyType,
 						WhenScaled:  appsv1beta2.RetainPersistentVolumeClaimRetentionPolicyType,
@@ -513,7 +504,7 @@ func TestSetDefaultDeployment(t *testing.T) {
 			original: &appsv1beta2.Deployment{},
 			expected: &appsv1beta2.Deployment{
 				Spec: appsv1beta2.DeploymentSpec{
-					Replicas: utilpointer.Int32(1),
+					Replicas: ptr.To[int32](1),
 					Strategy: appsv1beta2.DeploymentStrategy{
 						Type: appsv1beta2.RollingUpdateDeploymentStrategyType,
 						RollingUpdate: &appsv1beta2.RollingUpdateDeployment{
@@ -521,8 +512,8 @@ func TestSetDefaultDeployment(t *testing.T) {
 							MaxUnavailable: &defaultIntOrString,
 						},
 					},
-					RevisionHistoryLimit:    utilpointer.Int32(10),
-					ProgressDeadlineSeconds: utilpointer.Int32(600),
+					RevisionHistoryLimit:    ptr.To[int32](10),
+					ProgressDeadlineSeconds: ptr.To[int32](600),
 					Template:                defaultTemplate,
 				},
 			},
@@ -530,7 +521,7 @@ func TestSetDefaultDeployment(t *testing.T) {
 		{
 			original: &appsv1beta2.Deployment{
 				Spec: appsv1beta2.DeploymentSpec{
-					Replicas: utilpointer.Int32(5),
+					Replicas: ptr.To[int32](5),
 					Strategy: appsv1beta2.DeploymentStrategy{
 						RollingUpdate: &appsv1beta2.RollingUpdateDeployment{
 							MaxSurge: &differentIntOrString,
@@ -540,7 +531,7 @@ func TestSetDefaultDeployment(t *testing.T) {
 			},
 			expected: &appsv1beta2.Deployment{
 				Spec: appsv1beta2.DeploymentSpec{
-					Replicas: utilpointer.Int32(5),
+					Replicas: ptr.To[int32](5),
 					Strategy: appsv1beta2.DeploymentStrategy{
 						Type: appsv1beta2.RollingUpdateDeploymentStrategyType,
 						RollingUpdate: &appsv1beta2.RollingUpdateDeployment{
@@ -548,8 +539,8 @@ func TestSetDefaultDeployment(t *testing.T) {
 							MaxUnavailable: &defaultIntOrString,
 						},
 					},
-					RevisionHistoryLimit:    utilpointer.Int32(10),
-					ProgressDeadlineSeconds: utilpointer.Int32(600),
+					RevisionHistoryLimit:    ptr.To[int32](10),
+					ProgressDeadlineSeconds: ptr.To[int32](600),
 					Template:                defaultTemplate,
 				},
 			},
@@ -557,7 +548,7 @@ func TestSetDefaultDeployment(t *testing.T) {
 		{
 			original: &appsv1beta2.Deployment{
 				Spec: appsv1beta2.DeploymentSpec{
-					Replicas: utilpointer.Int32(3),
+					Replicas: ptr.To[int32](3),
 					Strategy: appsv1beta2.DeploymentStrategy{
 						Type:          appsv1beta2.RollingUpdateDeploymentStrategyType,
 						RollingUpdate: nil,
@@ -566,7 +557,7 @@ func TestSetDefaultDeployment(t *testing.T) {
 			},
 			expected: &appsv1beta2.Deployment{
 				Spec: appsv1beta2.DeploymentSpec{
-					Replicas: utilpointer.Int32(3),
+					Replicas: ptr.To[int32](3),
 					Strategy: appsv1beta2.DeploymentStrategy{
 						Type: appsv1beta2.RollingUpdateDeploymentStrategyType,
 						RollingUpdate: &appsv1beta2.RollingUpdateDeployment{
@@ -574,8 +565,8 @@ func TestSetDefaultDeployment(t *testing.T) {
 							MaxUnavailable: &defaultIntOrString,
 						},
 					},
-					RevisionHistoryLimit:    utilpointer.Int32(10),
-					ProgressDeadlineSeconds: utilpointer.Int32(600),
+					RevisionHistoryLimit:    ptr.To[int32](10),
+					ProgressDeadlineSeconds: ptr.To[int32](600),
 					Template:                defaultTemplate,
 				},
 			},
@@ -583,21 +574,21 @@ func TestSetDefaultDeployment(t *testing.T) {
 		{
 			original: &appsv1beta2.Deployment{
 				Spec: appsv1beta2.DeploymentSpec{
-					Replicas: utilpointer.Int32(5),
+					Replicas: ptr.To[int32](5),
 					Strategy: appsv1beta2.DeploymentStrategy{
 						Type: appsv1beta2.RecreateDeploymentStrategyType,
 					},
-					RevisionHistoryLimit: utilpointer.Int32(0),
+					RevisionHistoryLimit: ptr.To[int32](0),
 				},
 			},
 			expected: &appsv1beta2.Deployment{
 				Spec: appsv1beta2.DeploymentSpec{
-					Replicas: utilpointer.Int32(5),
+					Replicas: ptr.To[int32](5),
 					Strategy: appsv1beta2.DeploymentStrategy{
 						Type: appsv1beta2.RecreateDeploymentStrategyType,
 					},
-					RevisionHistoryLimit:    utilpointer.Int32(0),
-					ProgressDeadlineSeconds: utilpointer.Int32(600),
+					RevisionHistoryLimit:    ptr.To[int32](0),
+					ProgressDeadlineSeconds: ptr.To[int32](600),
 					Template:                defaultTemplate,
 				},
 			},
@@ -605,22 +596,22 @@ func TestSetDefaultDeployment(t *testing.T) {
 		{
 			original: &appsv1beta2.Deployment{
 				Spec: appsv1beta2.DeploymentSpec{
-					Replicas: utilpointer.Int32(5),
+					Replicas: ptr.To[int32](5),
 					Strategy: appsv1beta2.DeploymentStrategy{
 						Type: appsv1beta2.RecreateDeploymentStrategyType,
 					},
-					ProgressDeadlineSeconds: utilpointer.Int32(30),
-					RevisionHistoryLimit:    utilpointer.Int32(2),
+					ProgressDeadlineSeconds: ptr.To[int32](30),
+					RevisionHistoryLimit:    ptr.To[int32](2),
 				},
 			},
 			expected: &appsv1beta2.Deployment{
 				Spec: appsv1beta2.DeploymentSpec{
-					Replicas: utilpointer.Int32(5),
+					Replicas: ptr.To[int32](5),
 					Strategy: appsv1beta2.DeploymentStrategy{
 						Type: appsv1beta2.RecreateDeploymentStrategyType,
 					},
-					ProgressDeadlineSeconds: utilpointer.Int32(30),
-					RevisionHistoryLimit:    utilpointer.Int32(2),
+					ProgressDeadlineSeconds: ptr.To[int32](30),
+					RevisionHistoryLimit:    ptr.To[int32](2),
 					Template:                defaultTemplate,
 				},
 			},
@@ -677,7 +668,7 @@ func TestSetDefaultReplicaSetReplicas(t *testing.T) {
 		{
 			rs: appsv1beta2.ReplicaSet{
 				Spec: appsv1beta2.ReplicaSetSpec{
-					Replicas: utilpointer.Int32(0),
+					Replicas: ptr.To[int32](0),
 					Template: v1.PodTemplateSpec{
 						ObjectMeta: metav1.ObjectMeta{
 							Labels: map[string]string{
@@ -692,7 +683,7 @@ func TestSetDefaultReplicaSetReplicas(t *testing.T) {
 		{
 			rs: appsv1beta2.ReplicaSet{
 				Spec: appsv1beta2.ReplicaSetSpec{
-					Replicas: utilpointer.Int32(3),
+					Replicas: ptr.To[int32](3),
 					Template: v1.PodTemplateSpec{
 						ObjectMeta: metav1.ObjectMeta{
 							Labels: map[string]string{
@@ -735,7 +726,7 @@ func TestDefaultRequestIsNotSetForReplicaSet(t *testing.T) {
 	}
 	rs := &appsv1beta2.ReplicaSet{
 		Spec: appsv1beta2.ReplicaSetSpec{
-			Replicas: utilpointer.Int32(3),
+			Replicas: ptr.To[int32](3),
 			Template: v1.PodTemplateSpec{
 				ObjectMeta: metav1.ObjectMeta{
 					Labels: map[string]string{
