@@ -33,7 +33,6 @@ import (
 	"k8s.io/client-go/tools/cache"
 	"k8s.io/client-go/tools/events"
 	"k8s.io/kubernetes/pkg/registry/core/service/ipallocator"
-	netutils "k8s.io/utils/net"
 )
 
 var (
@@ -43,8 +42,9 @@ var (
 
 type fakeRepair struct {
 	*RepairIPAddress
-	serviceStore   cache.Store
-	ipAddressStore cache.Store
+	serviceStore     cache.Store
+	ipAddressStore   cache.Store
+	serviceCIDRStore cache.Store
 }
 
 func newFakeRepair() (*fake.Clientset, *fakeRepair) {
@@ -54,6 +54,9 @@ func newFakeRepair() (*fake.Clientset, *fakeRepair) {
 	serviceInformer := informerFactory.Core().V1().Services()
 	serviceIndexer := serviceInformer.Informer().GetIndexer()
 
+	serviceCIDRInformer := informerFactory.Networking().V1alpha1().ServiceCIDRs()
+	serviceCIDRIndexer := serviceCIDRInformer.Informer().GetIndexer()
+
 	ipInformer := informerFactory.Networking().V1alpha1().IPAddresses()
 	ipIndexer := ipInformer.Informer().GetIndexer()
 
@@ -72,22 +75,13 @@ func newFakeRepair() (*fake.Clientset, *fakeRepair) {
 		return false, &networkingv1alpha1.IPAddress{}, err
 	}))
 
-	_, primary, err := netutils.ParseCIDRSloppy(serviceCIDRv4)
-	if err != nil {
-		panic(err)
-	}
-	_, secondary, err := netutils.ParseCIDRSloppy(serviceCIDRv6)
-	if err != nil {
-		panic(err)
-	}
 	r := NewRepairIPAddress(0*time.Second,
 		fakeClient,
-		primary,
-		secondary,
 		serviceInformer,
+		serviceCIDRInformer,
 		ipInformer,
 	)
-	return fakeClient, &fakeRepair{r, serviceIndexer, ipIndexer}
+	return fakeClient, &fakeRepair{r, serviceIndexer, ipIndexer, serviceCIDRIndexer}
 }
 
 func TestRepairServiceIP(t *testing.T) {
@@ -95,6 +89,7 @@ func TestRepairServiceIP(t *testing.T) {
 		name        string
 		svcs        []*v1.Service
 		ipAddresses []*networkingv1alpha1.IPAddress
+		cidrs       []*networkingv1alpha1.ServiceCIDR
 		expectedIPs []string
 		actions     [][]string // verb and resource
 		events      []string
@@ -105,6 +100,9 @@ func TestRepairServiceIP(t *testing.T) {
 			ipAddresses: []*networkingv1alpha1.IPAddress{
 				newIPAddress("10.0.1.1", newService("test-svc", []string{"10.0.1.1"})),
 			},
+			cidrs: []*networkingv1alpha1.ServiceCIDR{
+				newServiceCIDR("kubernetes", serviceCIDRv4, serviceCIDRv6),
+			},
 			expectedIPs: []string{"10.0.1.1"},
 			actions:     [][]string{},
 			events:      []string{},
@@ -116,21 +114,45 @@ func TestRepairServiceIP(t *testing.T) {
 				newIPAddress("10.0.1.1", newService("test-svc", []string{"10.0.1.1"})),
 				newIPAddress("2001:db8::10", newService("test-svc", []string{"2001:db8::10"})),
 			},
+			cidrs: []*networkingv1alpha1.ServiceCIDR{
+				newServiceCIDR("kubernetes", serviceCIDRv4, serviceCIDRv6),
+			},
 			expectedIPs: []string{"10.0.1.1", "2001:db8::10"},
 			actions:     [][]string{},
 			events:      []string{},
 		},
+		{
+			name: "no changes needed dual stack multiple cidrs",
+			svcs: []*v1.Service{newService("test-svc", []string{"192.168.0.1", "2001:db8:a:b::10"})},
+			ipAddresses: []*networkingv1alpha1.IPAddress{
+				newIPAddress("192.168.0.1", newService("test-svc", []string{"192.168.0.1"})),
+				newIPAddress("2001:db8:a:b::10", newService("test-svc", []string{"2001:db8:a:b::10"})),
+			},
+			cidrs: []*networkingv1alpha1.ServiceCIDR{
+				newServiceCIDR("kubernetes", serviceCIDRv4, serviceCIDRv6),
+				newServiceCIDR("custom", "192.168.0.0/24", "2001:db8:a:b::/64"),
+			},
+			expectedIPs: []string{"192.168.0.1", "2001:db8:a:b::10"},
+			actions:     [][]string{},
+			events:      []string{},
+		},
 		// these two cases simulate migrating from bitmaps to IPAddress objects
 		{
-			name:        "create IPAddress single stack",
-			svcs:        []*v1.Service{newService("test-svc", []string{"10.0.1.1"})},
+			name: "create IPAddress single stack",
+			svcs: []*v1.Service{newService("test-svc", []string{"10.0.1.1"})},
+			cidrs: []*networkingv1alpha1.ServiceCIDR{
+				newServiceCIDR("kubernetes", serviceCIDRv4, serviceCIDRv6),
+			},
 			expectedIPs: []string{"10.0.1.1"},
 			actions:     [][]string{{"create", "ipaddresses"}},
 			events:      []string{"Warning ClusterIPNotAllocated Cluster IP [IPv4]: 10.0.1.1 is not allocated; repairing"},
 		},
 		{
-			name:        "create IPAddresses dual stack",
-			svcs:        []*v1.Service{newService("test-svc", []string{"10.0.1.1", "2001:db8::10"})},
+			name: "create IPAddresses dual stack",
+			svcs: []*v1.Service{newService("test-svc", []string{"10.0.1.1", "2001:db8::10"})},
+			cidrs: []*networkingv1alpha1.ServiceCIDR{
+				newServiceCIDR("kubernetes", serviceCIDRv4, serviceCIDRv6),
+			},
 			expectedIPs: []string{"10.0.1.1", "2001:db8::10"},
 			actions:     [][]string{{"create", "ipaddresses"}, {"create", "ipaddresses"}},
 			events: []string{
@@ -138,12 +160,26 @@ func TestRepairServiceIP(t *testing.T) {
 				"Warning ClusterIPNotAllocated Cluster IP [IPv6]: 2001:db8::10 is not allocated; repairing",
 			},
 		},
+		{
+			name: "create IPAddress single stack from secondary",
+			svcs: []*v1.Service{newService("test-svc", []string{"192.168.1.1"})},
+			cidrs: []*networkingv1alpha1.ServiceCIDR{
+				newServiceCIDR("kubernetes", serviceCIDRv4, serviceCIDRv6),
+				newServiceCIDR("custom", "192.168.1.0/24", ""),
+			},
+			expectedIPs: []string{"192.168.1.1"},
+			actions:     [][]string{{"create", "ipaddresses"}},
+			events:      []string{"Warning ClusterIPNotAllocated Cluster IP [IPv4]: 192.168.1.1 is not allocated; repairing"},
+		},
 		{
 			name: "reconcile IPAddress single stack wrong reference",
 			svcs: []*v1.Service{newService("test-svc", []string{"10.0.1.1"})},
 			ipAddresses: []*networkingv1alpha1.IPAddress{
 				newIPAddress("10.0.1.1", newService("test-svc2", []string{"10.0.1.1"})),
 			},
+			cidrs: []*networkingv1alpha1.ServiceCIDR{
+				newServiceCIDR("kubernetes", serviceCIDRv4, serviceCIDRv6),
+			},
 			expectedIPs: []string{"10.0.1.1"},
 			actions:     [][]string{{"delete", "ipaddresses"}, {"create", "ipaddresses"}},
 			events:      []string{"Warning ClusterIPNotAllocated the ClusterIP [IPv4]: 10.0.1.1 for Service bar/test-svc has a wrong reference; repairing"},
@@ -155,6 +191,9 @@ func TestRepairServiceIP(t *testing.T) {
 				newIPAddress("10.0.1.1", newService("test-svc2", []string{"10.0.1.1"})),
 				newIPAddress("2001:db8::10", newService("test-svc2", []string{"2001:db8::10"})),
 			},
+			cidrs: []*networkingv1alpha1.ServiceCIDR{
+				newServiceCIDR("kubernetes", serviceCIDRv4, serviceCIDRv6),
+			},
 			expectedIPs: []string{"10.0.1.1", "2001:db8::10"},
 			actions:     [][]string{{"delete", "ipaddresses"}, {"create", "ipaddresses"}, {"delete", "ipaddresses"}, {"create", "ipaddresses"}},
 			events: []string{
@@ -169,18 +208,85 @@ func TestRepairServiceIP(t *testing.T) {
 				newIPAddress("192.168.1.1", newService("test-svc", []string{"192.168.1.1"})),
 				newIPAddress("2001:db8::10", newService("test-svc", []string{"2001:db8::10"})),
 			},
+			cidrs: []*networkingv1alpha1.ServiceCIDR{
+				newServiceCIDR("kubernetes", serviceCIDRv4, serviceCIDRv6),
+			},
 			expectedIPs: []string{"2001:db8::10"},
 			actions:     [][]string{},
-			events:      []string{"Warning ClusterIPOutOfRange Cluster IP [IPv4]: 192.168.1.1 is not within the configured Service CIDR 10.0.0.0/16; please recreate service"},
+			events:      []string{"Warning ClusterIPOutOfRange Cluster IP [IPv4]: 192.168.1.1 is not within any configured Service CIDR; please recreate service"},
 		},
 		{
 			name: "one IP orphan",
 			ipAddresses: []*networkingv1alpha1.IPAddress{
 				newIPAddress("10.0.1.1", newService("test-svc", []string{"10.0.1.1"})),
 			},
+			cidrs: []*networkingv1alpha1.ServiceCIDR{
+				newServiceCIDR("kubernetes", serviceCIDRv4, serviceCIDRv6),
+			},
 			actions: [][]string{{"delete", "ipaddresses"}},
 			events:  []string{"Warning IPAddressNotAllocated IPAddress: 10.0.1.1 for Service bar/test-svc appears to have leaked: cleaning up"},
 		},
+		{
+			name: "one IP out of range matching the network address",
+			svcs: []*v1.Service{newService("test-svc", []string{"10.0.0.0"})},
+			ipAddresses: []*networkingv1alpha1.IPAddress{
+				newIPAddress("10.0.0.0", newService("test-svc", []string{"10.0.0.0"})),
+			},
+			cidrs: []*networkingv1alpha1.ServiceCIDR{
+				newServiceCIDR("kubernetes", serviceCIDRv4, serviceCIDRv6),
+			},
+			expectedIPs: []string{"10.0.0.0"},
+			actions:     [][]string{},
+			events:      []string{"Warning ClusterIPOutOfRange Cluster IP [IPv4]: 10.0.0.0 is not within any configured Service CIDR; please recreate service"},
+		},
+		{
+			name: "one IP out of range matching the broadcast address",
+			svcs: []*v1.Service{newService("test-svc", []string{"10.0.255.255"})},
+			ipAddresses: []*networkingv1alpha1.IPAddress{
+				newIPAddress("10.0.255.255", newService("test-svc", []string{"10.0.255.255"})),
+			},
+			cidrs: []*networkingv1alpha1.ServiceCIDR{
+				newServiceCIDR("kubernetes", serviceCIDRv4, serviceCIDRv6),
+			},
+			expectedIPs: []string{"10.0.255.255"},
+			actions:     [][]string{},
+			events:      []string{"Warning ClusterIPOutOfRange Cluster IP [IPv4]: 10.0.255.255 is not within any configured Service CIDR; please recreate service"},
+		},
+		{
+			name: "one IPv6 out of range matching the subnet address",
+			svcs: []*v1.Service{newService("test-svc", []string{"2001:db8::"})},
+			ipAddresses: []*networkingv1alpha1.IPAddress{
+				newIPAddress("2001:db8::", newService("test-svc", []string{"2001:db8::"})),
+			},
+			cidrs: []*networkingv1alpha1.ServiceCIDR{
+				newServiceCIDR("kubernetes", serviceCIDRv4, serviceCIDRv6),
+			},
+			expectedIPs: []string{"2001:db8::"},
+			actions:     [][]string{},
+			events:      []string{"Warning ClusterIPOutOfRange Cluster IP [IPv6]: 2001:db8:: is not within any configured Service CIDR; please recreate service"},
+		},
+		{
+			name: "one IPv6 matching the broadcast address",
+			svcs: []*v1.Service{newService("test-svc", []string{"2001:db8::ffff:ffff:ffff:ffff"})},
+			ipAddresses: []*networkingv1alpha1.IPAddress{
+				newIPAddress("2001:db8::ffff:ffff:ffff:ffff", newService("test-svc", []string{"2001:db8::ffff:ffff:ffff:ffff"})),
+			},
+			cidrs: []*networkingv1alpha1.ServiceCIDR{
+				newServiceCIDR("kubernetes", serviceCIDRv4, serviceCIDRv6),
+			},
+			expectedIPs: []string{"2001:db8::ffff:ffff:ffff:ffff"},
+		},
+		{
+			name: "one IP orphan matching the broadcast address",
+			ipAddresses: []*networkingv1alpha1.IPAddress{
+				newIPAddress("10.0.255.255", newService("test-svc", []string{"10.0.255.255"})),
+			},
+			cidrs: []*networkingv1alpha1.ServiceCIDR{
+				newServiceCIDR("kubernetes", serviceCIDRv4, serviceCIDRv6),
+			},
+			actions: [][]string{{"delete", "ipaddresses"}},
+			events:  []string{"Warning IPAddressNotAllocated IPAddress: 10.0.255.255 for Service bar/test-svc appears to have leaked: cleaning up"},
+		},
 		{
 			name: "Two IPAddresses referencing the same service",
 			svcs: []*v1.Service{newService("test-svc", []string{"10.0.1.1"})},
@@ -188,6 +294,9 @@ func TestRepairServiceIP(t *testing.T) {
 				newIPAddress("10.0.1.1", newService("test-svc", []string{"10.0.1.1"})),
 				newIPAddress("10.0.1.2", newService("test-svc", []string{"10.0.1.1"})),
 			},
+			cidrs: []*networkingv1alpha1.ServiceCIDR{
+				newServiceCIDR("kubernetes", serviceCIDRv4, serviceCIDRv6),
+			},
 			actions: [][]string{{"delete", "ipaddresses"}},
 			events:  []string{"Warning IPAddressWrongReference IPAddress: 10.0.1.2 for Service bar/test-svc has a wrong reference; cleaning up"},
 		},
@@ -200,7 +309,10 @@ func TestRepairServiceIP(t *testing.T) {
 			ipAddresses: []*networkingv1alpha1.IPAddress{
 				newIPAddress("10.0.1.1", newService("test-svc2", []string{"10.0.1.1"})),
 			},
-			events: []string{"Warning ClusterIPAlreadyAllocated Cluster IP [4]:10.0.1.1 was assigned to multiple services; please recreate service"},
+			cidrs: []*networkingv1alpha1.ServiceCIDR{
+				newServiceCIDR("kubernetes", serviceCIDRv4, serviceCIDRv6),
+			},
+			events: []string{"Warning ClusterIPAlreadyAllocated Cluster IP [IPv4]:10.0.1.1 was assigned to multiple services; please recreate service"},
 		},
 	}
 
@@ -208,9 +320,21 @@ func TestRepairServiceIP(t *testing.T) {
 		t.Run(test.name, func(t *testing.T) {
 
 			c, r := newFakeRepair()
+			// add cidrs
+			for _, cidr := range test.cidrs {
+				err := r.serviceCIDRStore.Add(cidr)
+				if err != nil {
+					t.Errorf("Unexpected error trying to add Service %v object: %v", cidr, err)
+				}
+			}
+			err := r.syncCIDRs()
+			if err != nil {
+				t.Fatal(err)
+			}
 			// override for testing
 			r.servicesSynced = func() bool { return true }
 			r.ipAddressSynced = func() bool { return true }
+			r.serviceCIDRSynced = func() bool { return true }
 			recorder := events.NewFakeRecorder(100)
 			r.recorder = recorder
 			for _, svc := range test.svcs {
@@ -228,7 +352,7 @@ func TestRepairServiceIP(t *testing.T) {
 				}
 			}
 
-			err := r.runOnce()
+			err = r.runOnce()
 			if err != nil {
 				t.Fatal(err)
 			}
@@ -402,6 +526,20 @@ func newService(name string, ips []string) *v1.Service {
 	return svc
 }
 
+func newServiceCIDR(name, primary, secondary string) *networkingv1alpha1.ServiceCIDR {
+	serviceCIDR := &networkingv1alpha1.ServiceCIDR{
+		ObjectMeta: metav1.ObjectMeta{
+			Name: name,
+		},
+		Spec: networkingv1alpha1.ServiceCIDRSpec{},
+	}
+	serviceCIDR.Spec.CIDRs = append(serviceCIDR.Spec.CIDRs, primary)
+	if secondary != "" {
+		serviceCIDR.Spec.CIDRs = append(serviceCIDR.Spec.CIDRs, secondary)
+	}
+	return serviceCIDR
+}
+
 func expectAction(t *testing.T, actions []k8stesting.Action, expected [][]string) {
 	t.Helper()
 	if len(actions) != len(expected) {
