@@ -20,9 +20,22 @@ limitations under the License.
 package filesystem
 
 import (
+	"fmt"
 	"os"
 )
 
+// IsUnixDomainSocket returns whether a given file is a AF_UNIX socket file
+func IsUnixDomainSocket(filePath string) (bool, error) {
+	fi, err := os.Stat(filePath)
+	if err != nil {
+		return false, fmt.Errorf("stat file %s failed: %v", filePath, err)
+	}
+	if fi.Mode()&os.ModeSocket == 0 {
+		return false, nil
+	}
+	return true, nil
+}
+
 // Chmod is the same as os.Chmod on Linux.
 func Chmod(name string, mode os.FileMode) error {
 	return os.Chmod(name, mode)
