@@ -505,6 +505,15 @@ func setMigrationAnnotation(migratedPlugins map[string](func() bool), nodeInfo *
 	return true
 }
 
+// Returns true if and only if new maxAttachLimit doesn't require CSINode update
+func keepAllocatableCount(driverInfoSpec storagev1.CSINodeDriver, maxAttachLimit int64) bool {
+	if maxAttachLimit == 0 {
+		return driverInfoSpec.Allocatable == nil || driverInfoSpec.Allocatable.Count == nil
+	}
+
+	return driverInfoSpec.Allocatable != nil && driverInfoSpec.Allocatable.Count != nil && int64(*driverInfoSpec.Allocatable.Count) == maxAttachLimit
+}
+
 func (nim *nodeInfoManager) installDriverToCSINode(
 	nodeInfo *storagev1.CSINode,
 	driverName string,
@@ -517,10 +526,7 @@ func (nim *nodeInfoManager) installDriverToCSINode(
 		return fmt.Errorf("error getting CSI client")
 	}
 
-	topologyKeys := make(sets.String)
-	for k := range topology {
-		topologyKeys.Insert(k)
-	}
+	topologyKeys := sets.StringKeySet(topology)
 
 	specModified := true
 	// Clone driver list, omitting the driver that matches the given driverName
@@ -528,7 +534,8 @@ func (nim *nodeInfoManager) installDriverToCSINode(
 	for _, driverInfoSpec := range nodeInfo.Spec.Drivers {
 		if driverInfoSpec.Name == driverName {
 			if driverInfoSpec.NodeID == driverNodeID &&
-				sets.NewString(driverInfoSpec.TopologyKeys...).Equal(topologyKeys) {
+				sets.NewString(driverInfoSpec.TopologyKeys...).Equal(topologyKeys) &&
+				keepAllocatableCount(driverInfoSpec, maxAttachLimit) {
 				specModified = false
 			}
 		} else {
@@ -557,7 +564,7 @@ func (nim *nodeInfoManager) installDriverToCSINode(
 		}
 		m := int32(maxAttachLimit)
 		driverSpec.Allocatable = &storagev1.VolumeNodeResources{Count: &m}
-	} else {
+	} else if maxAttachLimit != 0 {
 		klog.Errorf("Invalid attach limit value %d cannot be added to CSINode object for %q", maxAttachLimit, driverName)
 	}
 
