@@ -27,6 +27,7 @@ import (
 	"k8s.io/apimachinery/pkg/types"
 	"k8s.io/apimachinery/pkg/util/uuid"
 	"k8s.io/client-go/kubernetes/fake"
+	"k8s.io/klog/v2/ktesting"
 	"k8s.io/kubernetes/pkg/volume"
 	volumetest "k8s.io/kubernetes/pkg/volume/testing"
 	"k8s.io/kubernetes/pkg/volume/util/hostutil"
@@ -111,7 +112,8 @@ func TestDeleter(t *testing.T) {
 	if err != nil {
 		t.Fatal("Can't find the plugin by name")
 	}
-	deleter, err := plug.NewDeleter(spec)
+	logger, _ := ktesting.NewTestContext(t)
+	deleter, err := plug.NewDeleter(logger, spec)
 	if err != nil {
 		t.Errorf("Failed to make a new Deleter: %v", err)
 	}
@@ -135,13 +137,13 @@ func TestDeleterTempDir(t *testing.T) {
 		"not-tmp":  {true, "/nottmp"},
 		"good-tmp": {false, "/tmp/scratch"},
 	}
-
+	logger, _ := ktesting.NewTestContext(t)
 	for name, test := range tests {
 		plugMgr := volume.VolumePluginMgr{}
 		plugMgr.InitPlugins(ProbeVolumePlugins(volume.VolumeConfig{}), nil /* prober */, volumetest.NewFakeKubeletVolumeHost(t, "/tmp/fake", nil, nil))
 		spec := &volume.Spec{PersistentVolume: &v1.PersistentVolume{Spec: v1.PersistentVolumeSpec{PersistentVolumeSource: v1.PersistentVolumeSource{HostPath: &v1.HostPathVolumeSource{Path: test.path}}}}}
 		plug, _ := plugMgr.FindDeletablePluginBySpec(spec)
-		deleter, _ := plug.NewDeleter(spec)
+		deleter, _ := plug.NewDeleter(logger, spec)
 		err := deleter.Delete()
 		if err == nil && test.expectedFailure {
 			t.Errorf("Expected failure for test '%s' but got nil err", name)
@@ -167,7 +169,8 @@ func TestProvisioner(t *testing.T) {
 		PVC:                           volumetest.CreateTestPVC("1Gi", []v1.PersistentVolumeAccessMode{v1.ReadWriteOnce}),
 		PersistentVolumeReclaimPolicy: v1.PersistentVolumeReclaimDelete,
 	}
-	creator, err := plug.NewProvisioner(options)
+	logger, _ := ktesting.NewTestContext(t)
+	creator, err := plug.NewProvisioner(logger, options)
 	if err != nil {
 		t.Fatalf("Failed to make a new Provisioner: %v", err)
 	}
