@@ -34,12 +34,12 @@ import (
 	"testing"
 	"time"
 
+	"github.com/google/go-cmp/cmp"
 	"golang.org/x/net/http2"
 
 	apierrors "k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/runtime/schema"
-	"k8s.io/apimachinery/pkg/util/diff"
 	"k8s.io/apimachinery/pkg/util/runtime"
 	"k8s.io/apimachinery/pkg/util/sets"
 	"k8s.io/apiserver/pkg/endpoints/request"
@@ -141,7 +141,7 @@ func TestTimeout(t *testing.T) {
 		t.Fatal(err)
 	}
 	if res.StatusCode != http.StatusGatewayTimeout {
-		t.Errorf("got res.StatusCode %d; expected %d", res.StatusCode, http.StatusServiceUnavailable)
+		t.Errorf("got res.StatusCode %d; expected %d", res.StatusCode, http.StatusGatewayTimeout)
 	}
 	body, _ = ioutil.ReadAll(res.Body)
 	status := &metav1.Status{}
@@ -149,7 +149,7 @@ func TestTimeout(t *testing.T) {
 		t.Fatal(err)
 	}
 	if !reflect.DeepEqual(status, &timeoutErr.ErrStatus) {
-		t.Errorf("unexpected object: %s", diff.ObjectReflectDiff(&timeoutErr.ErrStatus, status))
+		t.Errorf("unexpected object: %s", cmp.Diff(&timeoutErr.ErrStatus, status))
 	}
 	if record.Count() != 1 {
 		t.Errorf("did not invoke record method: %#v", record)
@@ -217,6 +217,7 @@ func TestTimeoutHeaders(t *testing.T) {
 		})
 	}
 
+	postTimeoutCh := make(chan struct{})
 	ts := httptest.NewServer(
 		withDeadline(
 			WithTimeout(
@@ -224,13 +225,18 @@ func TestTimeoutHeaders(t *testing.T) {
 					h := w.Header()
 					// trigger the timeout
 					cancel()
-					// mutate response Headers
-					for j := 0; j < 1000; j++ {
-						h.Set("Test", "post")
+					// keep mutating response Headers until the request times out
+					for {
+						select {
+						case <-postTimeoutCh:
+							return
+						default:
+							h.Set("Test", "post")
+						}
 					}
 				}),
 				func(req *http.Request) (*http.Request, bool, func(), *apierrors.StatusError) {
-					return req, false, func() {}, apierrors.NewServerTimeout(schema.GroupResource{Group: "foo", Resource: "bar"}, "get", 0)
+					return req, false, func() { close(postTimeoutCh) }, apierrors.NewServerTimeout(schema.GroupResource{Group: "foo", Resource: "bar"}, "get", 0)
 				},
 			),
 		),
@@ -242,7 +248,7 @@ func TestTimeoutHeaders(t *testing.T) {
 		t.Fatal(err)
 	}
 	if res.StatusCode != http.StatusGatewayTimeout {
-		t.Errorf("got res.StatusCode %d; expected %d", res.StatusCode, http.StatusServiceUnavailable)
+		t.Errorf("got res.StatusCode %d; expected %d", res.StatusCode, http.StatusGatewayTimeout)
 	}
 	res.Body.Close()
 }
@@ -266,6 +272,8 @@ func TestTimeoutRequestHeaders(t *testing.T) {
 		})
 	}
 
+	testDone := make(chan struct{})
+	defer close(testDone)
 	ts := httptest.NewServer(
 		withDeadline(
 			WithTimeoutForNonLongRunningRequests(
@@ -274,8 +282,13 @@ func TestTimeoutRequestHeaders(t *testing.T) {
 					cancel()
 					// mutate request Headers
 					// Authorization filter does it for example
-					for j := 0; j < 10000; j++ {
-						req.Header.Set("Test", "post")
+					for {
+						select {
+						case <-testDone:
+							return
+						default:
+							req.Header.Set("Test", "post")
+						}
 					}
 				}),
 				func(r *http.Request, requestInfo *request.RequestInfo) bool {
@@ -295,8 +308,8 @@ func TestTimeoutRequestHeaders(t *testing.T) {
 	if err != nil {
 		t.Fatal(err)
 	}
-	if res.StatusCode != http.StatusGatewayTimeout {
-		t.Errorf("got res.StatusCde %d; expected %d", res.StatusCode, http.StatusServiceUnavailable)
+	if actual, expected := res.StatusCode, http.StatusGatewayTimeout; actual != expected {
+		t.Errorf("got status code %d; expected %d", actual, expected)
 	}
 	res.Body.Close()
 }
@@ -317,6 +330,8 @@ func TestTimeoutWithLogging(t *testing.T) {
 		})
 	}
 
+	testDone := make(chan struct{})
+	defer close(testDone)
 	ts := httptest.NewServer(
 		WithHTTPLogging(
 			withDeadline(
@@ -326,8 +341,13 @@ func TestTimeoutWithLogging(t *testing.T) {
 						cancel()
 						// mutate request Headers
 						// Authorization filter does it for example
-						for j := 0; j < 10000; j++ {
-							req.Header.Set("Test", "post")
+						for {
+							select {
+							case <-testDone:
+								return
+							default:
+								req.Header.Set("Test", "post")
+							}
 						}
 					}),
 					func(r *http.Request, requestInfo *request.RequestInfo) bool {
@@ -348,8 +368,8 @@ func TestTimeoutWithLogging(t *testing.T) {
 	if err != nil {
 		t.Fatal(err)
 	}
-	if res.StatusCode != http.StatusGatewayTimeout {
-		t.Errorf("got res.StatusCode %d; expected %d", res.StatusCode, http.StatusServiceUnavailable)
+	if actual, expected := res.StatusCode, http.StatusGatewayTimeout; actual != expected {
+		t.Errorf("got status code %d; expected %d", actual, expected)
 	}
 	res.Body.Close()
 }
