@@ -29,6 +29,7 @@ KUBE_ROOT=$(dirname "${BASH_SOURCE[0]}")/../..
 # source "${KUBE_ROOT}/hack/lib/test.sh"
 source "${KUBE_ROOT}/test/cmd/apply.sh"
 source "${KUBE_ROOT}/test/cmd/apps.sh"
+source "${KUBE_ROOT}/test/cmd/auth_whoami.sh"
 source "${KUBE_ROOT}/test/cmd/authentication.sh"
 source "${KUBE_ROOT}/test/cmd/authorization.sh"
 source "${KUBE_ROOT}/test/cmd/batch.sh"
@@ -41,9 +42,11 @@ source "${KUBE_ROOT}/test/cmd/debug.sh"
 source "${KUBE_ROOT}/test/cmd/delete.sh"
 source "${KUBE_ROOT}/test/cmd/diff.sh"
 source "${KUBE_ROOT}/test/cmd/discovery.sh"
+source "${KUBE_ROOT}/test/cmd/events.sh"
 source "${KUBE_ROOT}/test/cmd/exec.sh"
 source "${KUBE_ROOT}/test/cmd/generic-resources.sh"
 source "${KUBE_ROOT}/test/cmd/get.sh"
+source "${KUBE_ROOT}/test/cmd/help.sh"
 source "${KUBE_ROOT}/test/cmd/kubeconfig.sh"
 source "${KUBE_ROOT}/test/cmd/node-management.sh"
 source "${KUBE_ROOT}/test/cmd/plugins.sh"
@@ -67,15 +70,15 @@ KUBELET_HEALTHZ_PORT=${KUBELET_HEALTHZ_PORT:-10248}
 SECURE_CTLRMGR_PORT=${SECURE_CTLRMGR_PORT:-10257}
 PROXY_HOST=127.0.0.1 # kubectl only serves on localhost.
 
-IMAGE_NGINX="k8s.gcr.io/nginx:1.7.9"
-export IMAGE_DEPLOYMENT_R1="k8s.gcr.io/nginx:test-cmd"  # deployment-revision1.yaml
+IMAGE_NGINX="registry.k8s.io/nginx:1.7.9"
+export IMAGE_DEPLOYMENT_R1="registry.k8s.io/nginx:test-cmd"  # deployment-revision1.yaml
 export IMAGE_DEPLOYMENT_R2="$IMAGE_NGINX"  # deployment-revision2.yaml
-export IMAGE_PERL="k8s.gcr.io/perl"
-export IMAGE_PAUSE_V2="k8s.gcr.io/pause:2.0"
-export IMAGE_DAEMONSET_R2="k8s.gcr.io/pause:latest"
-export IMAGE_DAEMONSET_R2_2="k8s.gcr.io/nginx:test-cmd"  # rollingupdate-daemonset-rv2.yaml
-export IMAGE_STATEFULSET_R1="k8s.gcr.io/nginx-slim:0.7"
-export IMAGE_STATEFULSET_R2="k8s.gcr.io/nginx-slim:0.8"
+export IMAGE_PERL="registry.k8s.io/perl"
+export IMAGE_PAUSE_V2="registry.k8s.io/pause:2.0"
+export IMAGE_DAEMONSET_R2="registry.k8s.io/pause:latest"
+export IMAGE_DAEMONSET_R2_2="registry.k8s.io/nginx:test-cmd"  # rollingupdate-daemonset-rv2.yaml
+export IMAGE_STATEFULSET_R1="registry.k8s.io/nginx-slim:0.7"
+export IMAGE_STATEFULSET_R2="registry.k8s.io/nginx-slim:0.8"
 
 # Expose kubectl directly for readability
 PATH="${KUBE_OUTPUT_HOSTBIN}":$PATH
@@ -84,18 +87,19 @@ PATH="${KUBE_OUTPUT_HOSTBIN}":$PATH
 clusterroles="clusterroles"
 configmaps="configmaps"
 csr="csr"
+cronjob="cronjobs"
 deployments="deployments"
 namespaces="namespaces"
 nodes="nodes"
 persistentvolumeclaims="persistentvolumeclaims"
 persistentvolumes="persistentvolumes"
 pods="pods"
-podsecuritypolicies="podsecuritypolicies"
 podtemplates="podtemplates"
 replicasets="replicasets"
 replicationcontrollers="replicationcontrollers"
 roles="roles"
 secrets="secrets"
+selfsubjectreviews="selfsubjectreviews"
 serviceaccounts="serviceaccounts"
 services="services"
 statefulsets="statefulsets"
@@ -174,7 +178,7 @@ fi
 function stop-proxy()
 {
   [[ -n "${PROXY_PORT-}" ]] && kube::log::status "Stopping proxy on port ${PROXY_PORT}"
-  [[ -n "${PROXY_PID-}" ]] && kill "${PROXY_PID}" 1>&2 2>/dev/null
+  [[ -n "${PROXY_PID-}" ]] && kill -9 "${PROXY_PID}" 1>&2 2>/dev/null
   [[ -n "${PROXY_PORT_FILE-}" ]] && rm -f "${PROXY_PORT_FILE}"
   PROXY_PID=
   PROXY_PORT=
@@ -223,10 +227,10 @@ function start-proxy()
 
 function cleanup()
 {
-  [[ -n "${APISERVER_PID-}" ]] && kill "${APISERVER_PID}" 1>&2 2>/dev/null
-  [[ -n "${CTLRMGR_PID-}" ]] && kill "${CTLRMGR_PID}" 1>&2 2>/dev/null
-  [[ -n "${KUBELET_PID-}" ]] && kill "${KUBELET_PID}" 1>&2 2>/dev/null
   stop-proxy
+  [[ -n "${CTLRMGR_PID-}" ]] && kill -9 "${CTLRMGR_PID}" 1>&2 2>/dev/null
+  [[ -n "${KUBELET_PID-}" ]] && kill -9 "${KUBELET_PID}" 1>&2 2>/dev/null
+  [[ -n "${APISERVER_PID-}" ]] && kill -9 "${APISERVER_PID}" 1>&2 2>/dev/null
 
   kube::etcd::cleanup
   rm -rf "${KUBE_TEMP}"
@@ -502,7 +506,25 @@ runTests() {
   # Assert short name     #
   #########################
 
-  record_command run_assert_short_name_tests
+  if kube::test::if_supports_resource "${customresourcedefinitions}" && kube::test::if_supports_resource "${pods}" && kube::test::if_supports_resource "${configmaps}" ; then
+    record_command run_assert_short_name_tests
+  fi
+
+  #########################
+  # Assert singular name  #
+  #########################
+
+  if kube::test::if_supports_resource "${customresourcedefinitions}" && kube::test::if_supports_resource "${pods}" ; then
+    record_command run_assert_singular_name_tests
+  fi
+
+  #########################
+  # Ambiguous short name  #
+  #########################
+
+  if kube::test::if_supports_resource "${customresourcedefinitions}" ; then
+    record_command run_ambiguous_shortname_tests
+  fi
 
   #########################
   # Assert categories     #
@@ -554,6 +576,20 @@ runTests() {
     record_command run_kubectl_get_tests
   fi
 
+  ################
+  # Kubectl help #
+  ################
+
+  record_command run_kubectl_help_tests
+
+  ##################
+  # Kubectl events #
+  ##################
+
+  if kube::test::if_supports_resource "${cronjob}" ; then
+    record_command run_kubectl_events_tests
+  fi
+
   ################
   # Kubectl exec #
   ################
@@ -590,6 +626,13 @@ runTests() {
     record_command run_kubectl_delete_allnamespaces_tests
   fi
 
+  ######################
+  # Delete --interactive   #
+  ######################
+  if kube::test::if_supports_resource "${configmaps}" ; then
+    record_command run_kubectl_delete_interactive_tests
+  fi
+
   ##################
   # Global timeout #
   ##################
@@ -790,6 +833,10 @@ runTests() {
   record_command run_exec_credentials_tests
   record_command run_exec_credentials_interactive_tests
 
+  if kube::test::if_supports_resource "${selfsubjectreviews}" ; then
+    record_command run_kubectl_auth_whoami_tests
+  fi
+
   ########################
   # authorization.k8s.io #
   ########################
@@ -869,6 +916,8 @@ runTests() {
     kubectl delete "${kube_flags[@]}" rolebindings,role,clusterroles,clusterrolebindings -n some-other-random -l test-cmd=auth
   fi
 
+
+
   #####################
   # Retrieve multiple #
   #####################
@@ -934,7 +983,7 @@ runTests() {
   # Kubectl deprecated APIs  #
   ############################
 
-  if kube::test::if_supports_resource "${podsecuritypolicies}" ; then
+  if kube::test::if_supports_resource "${customresourcedefinitions}" ; then
     record_command run_deprecated_api_tests
   fi
 
@@ -985,9 +1034,17 @@ runTests() {
   ####################
   if kube::test::if_supports_resource "${pods}" ; then
     record_command run_kubectl_debug_pod_tests
+    record_command run_kubectl_debug_general_tests
+    record_command run_kubectl_debug_baseline_tests
+    record_command run_kubectl_debug_restricted_tests
+    record_command run_kubectl_debug_netadmin_tests
   fi
   if kube::test::if_supports_resource "${nodes}" ; then
     record_command run_kubectl_debug_node_tests
+    record_command run_kubectl_debug_general_node_tests
+    record_command run_kubectl_debug_baseline_node_tests
+    record_command run_kubectl_debug_restricted_node_tests
+    record_command run_kubectl_debug_netadmin_node_tests
   fi
 
   cleanup_tests
