@@ -18,10 +18,10 @@ limitations under the License.
  * This file defines various csi volume test drivers for TestSuites.
  *
  * There are two ways, how to prepare test drivers:
- * 1) With containerized server (NFS, Ceph, Gluster, iSCSI, ...)
+ * 1) With containerized server (NFS, Ceph, iSCSI, ...)
  * It creates a server pod which defines one volume for the tests.
  * These tests work only when privileged containers are allowed, exporting
- * various filesystems (NFS, GlusterFS, ...) usually needs some mounting or
+ * various filesystems (ex: NFS) usually needs some mounting or
  * other privileged magic in the server pod.
  *
  * Note that the server containers are for testing purposes only and should not
@@ -45,7 +45,7 @@ import (
 	"sync"
 	"time"
 
-	"github.com/onsi/ginkgo"
+	"github.com/onsi/ginkgo/v2"
 	spb "google.golang.org/genproto/googleapis/rpc/status"
 	"google.golang.org/grpc/codes"
 	grpcstatus "google.golang.org/grpc/status"
@@ -59,9 +59,9 @@ import (
 	"k8s.io/apimachinery/pkg/apis/meta/v1/unstructured"
 	"k8s.io/apimachinery/pkg/util/sets"
 	"k8s.io/apimachinery/pkg/util/wait"
-	"k8s.io/client-go/kubernetes"
 	clientset "k8s.io/client-go/kubernetes"
 	"k8s.io/klog/v2"
+	"k8s.io/kubernetes/test/e2e/feature"
 	"k8s.io/kubernetes/test/e2e/framework"
 	e2enode "k8s.io/kubernetes/test/e2e/framework/node"
 	e2epod "k8s.io/kubernetes/test/e2e/framework/pod"
@@ -97,7 +97,6 @@ func initHostPathCSIDriver(name string, capabilities map[storageframework.Capabi
 	return &hostpathCSIDriver{
 		driverInfo: storageframework.DriverInfo{
 			Name:        name,
-			FeatureTag:  "",
 			MaxFileSize: storageframework.FileSizeMedium,
 			SupportedFsType: sets.NewString(
 				"", // Default fsType
@@ -140,14 +139,18 @@ var _ storageframework.EphemeralTestDriver = &hostpathCSIDriver{}
 // InitHostPathCSIDriver returns hostpathCSIDriver that implements TestDriver interface
 func InitHostPathCSIDriver() storageframework.TestDriver {
 	capabilities := map[storageframework.Capability]bool{
-		storageframework.CapPersistence:         true,
-		storageframework.CapSnapshotDataSource:  true,
-		storageframework.CapMultiPODs:           true,
-		storageframework.CapBlock:               true,
-		storageframework.CapPVCDataSource:       true,
-		storageframework.CapControllerExpansion: true,
-		storageframework.CapOnlineExpansion:     true,
-		storageframework.CapSingleNodeVolume:    true,
+		storageframework.CapPersistence:                    true,
+		storageframework.CapSnapshotDataSource:             true,
+		storageframework.CapMultiPODs:                      true,
+		storageframework.CapBlock:                          true,
+		storageframework.CapPVCDataSource:                  true,
+		storageframework.CapControllerExpansion:            true,
+		storageframework.CapOfflineExpansion:               true,
+		storageframework.CapOnlineExpansion:                true,
+		storageframework.CapSingleNodeVolume:               true,
+		storageframework.CapReadWriteOncePod:               true,
+		storageframework.CapMultiplePVsSameID:              true,
+		storageframework.CapFSResizeFromSourceNotSupported: true,
 
 		// This is needed for the
 		// testsuites/volumelimits.go `should support volume limits`
@@ -182,7 +185,7 @@ func (h *hostpathCSIDriver) SkipUnsupportedTest(pattern storageframework.TestPat
 	}
 }
 
-func (h *hostpathCSIDriver) GetDynamicProvisionStorageClass(config *storageframework.PerTestConfig, fsType string) *storagev1.StorageClass {
+func (h *hostpathCSIDriver) GetDynamicProvisionStorageClass(ctx context.Context, config *storageframework.PerTestConfig, fsType string) *storagev1.StorageClass {
 	provisioner := config.GetUniqueDriverName()
 	parameters := map[string]string{}
 	ns := config.Framework.Namespace.Name
@@ -198,25 +201,25 @@ func (h *hostpathCSIDriver) GetCSIDriverName(config *storageframework.PerTestCon
 	return config.GetUniqueDriverName()
 }
 
-func (h *hostpathCSIDriver) GetSnapshotClass(config *storageframework.PerTestConfig, parameters map[string]string) *unstructured.Unstructured {
+func (h *hostpathCSIDriver) GetSnapshotClass(ctx context.Context, config *storageframework.PerTestConfig, parameters map[string]string) *unstructured.Unstructured {
 	snapshotter := config.GetUniqueDriverName()
 	ns := config.Framework.Namespace.Name
 
 	return utils.GenerateSnapshotClassSpec(snapshotter, parameters, ns)
 }
 
-func (h *hostpathCSIDriver) PrepareTest(f *framework.Framework) (*storageframework.PerTestConfig, func()) {
+func (h *hostpathCSIDriver) PrepareTest(ctx context.Context, f *framework.Framework) *storageframework.PerTestConfig {
 	// Create secondary namespace which will be used for creating driver
-	driverNamespace := utils.CreateDriverNamespace(f)
+	driverNamespace := utils.CreateDriverNamespace(ctx, f)
 	driverns := driverNamespace.Name
 	testns := f.Namespace.Name
 
 	ginkgo.By(fmt.Sprintf("deploying %s driver", h.driverInfo.Name))
-	cancelLogging := utils.StartPodLogs(f, driverNamespace)
+	cancelLogging := utils.StartPodLogs(ctx, f, driverNamespace)
 	cs := f.ClientSet
 
 	// The hostpath CSI driver only works when everything runs on the same node.
-	node, err := e2enode.GetRandomReadySchedulableNode(cs)
+	node, err := e2enode.GetRandomReadySchedulableNode(ctx, cs)
 	framework.ExpectNoError(err)
 	config := &storageframework.PerTestConfig{
 		Driver:              h,
@@ -244,7 +247,7 @@ func (h *hostpathCSIDriver) PrepareTest(f *framework.Framework) (*storageframewo
 		NodeName:                 node.Name,
 	}
 
-	cleanup, err := utils.CreateFromManifests(config.Framework, driverNamespace, func(item interface{}) error {
+	err = utils.CreateFromManifests(ctx, config.Framework, driverNamespace, func(item interface{}) error {
 		if err := utils.PatchCSIDeployment(config.Framework, o, item); err != nil {
 			return err
 		}
@@ -282,33 +285,35 @@ func (h *hostpathCSIDriver) PrepareTest(f *framework.Framework) (*storageframewo
 		h.driverInfo.Name,
 		testns,
 		driverns,
-		cleanup,
 		cancelLogging)
+	ginkgo.DeferCleanup(cleanupFunc)
 
-	return config, cleanupFunc
+	return config
 }
 
 // mockCSI
 type mockCSIDriver struct {
-	driverInfo             storageframework.DriverInfo
-	manifests              []string
-	podInfo                *bool
-	storageCapacity        *bool
-	attachable             bool
-	attachLimit            int
-	enableTopology         bool
-	enableNodeExpansion    bool
-	hooks                  Hooks
-	tokenRequests          []storagev1.TokenRequest
-	requiresRepublish      *bool
-	fsGroupPolicy          *storagev1.FSGroupPolicy
-	enableVolumeMountGroup bool
-	embedded               bool
-	calls                  MockCSICalls
-	embeddedCSIDriver      *mockdriver.CSIDriver
+	driverInfo                    storageframework.DriverInfo
+	manifests                     []string
+	podInfo                       *bool
+	storageCapacity               *bool
+	attachable                    bool
+	attachLimit                   int
+	enableTopology                bool
+	enableNodeExpansion           bool
+	hooks                         Hooks
+	tokenRequests                 []storagev1.TokenRequest
+	requiresRepublish             *bool
+	fsGroupPolicy                 *storagev1.FSGroupPolicy
+	enableVolumeMountGroup        bool
+	embedded                      bool
+	calls                         MockCSICalls
+	embeddedCSIDriver             *mockdriver.CSIDriver
+	enableSELinuxMount            *bool
+	enableRecoverExpansionFailure bool
 
 	// Additional values set during PrepareTest
-	clientSet       kubernetes.Interface
+	clientSet       clientset.Interface
 	driverNamespace *v1.Namespace
 }
 
@@ -334,24 +339,26 @@ type MockCSITestDriver interface {
 
 	// GetCalls returns all currently observed gRPC calls. Only valid
 	// after PrepareTest.
-	GetCalls() ([]MockCSICall, error)
+	GetCalls(ctx context.Context) ([]MockCSICall, error)
 }
 
 // CSIMockDriverOpts defines options used for csi driver
 type CSIMockDriverOpts struct {
-	RegisterDriver         bool
-	DisableAttach          bool
-	PodInfo                *bool
-	StorageCapacity        *bool
-	AttachLimit            int
-	EnableTopology         bool
-	EnableResizing         bool
-	EnableNodeExpansion    bool
-	EnableSnapshot         bool
-	EnableVolumeMountGroup bool
-	TokenRequests          []storagev1.TokenRequest
-	RequiresRepublish      *bool
-	FSGroupPolicy          *storagev1.FSGroupPolicy
+	RegisterDriver                bool
+	DisableAttach                 bool
+	PodInfo                       *bool
+	StorageCapacity               *bool
+	AttachLimit                   int
+	EnableTopology                bool
+	EnableResizing                bool
+	EnableNodeExpansion           bool
+	EnableSnapshot                bool
+	EnableVolumeMountGroup        bool
+	TokenRequests                 []storagev1.TokenRequest
+	RequiresRepublish             *bool
+	FSGroupPolicy                 *storagev1.FSGroupPolicy
+	EnableSELinuxMount            *bool
+	EnableRecoverExpansionFailure bool
 
 	// Embedded defines whether the CSI mock driver runs
 	// inside the cluster (false, the default) or just a proxy
@@ -372,6 +379,7 @@ type MockCSICall struct {
 	Method  string
 	Request struct {
 		VolumeContext map[string]string `json:"volume_context"`
+		Secrets       map[string]string `json:"secrets"`
 	}
 	FullError struct {
 		Code    codes.Code `json:"code"`
@@ -394,7 +402,7 @@ func (c *MockCSICalls) Get() []MockCSICall {
 	return c.calls[:]
 }
 
-// Add appens one new call at the end.
+// Add appends one new call at the end.
 func (c *MockCSICalls) Add(call MockCSICall) {
 	c.mutex.Lock()
 	defer c.mutex.Unlock()
@@ -480,31 +488,33 @@ func InitMockCSIDriver(driverOpts CSIMockDriverOpts) MockCSITestDriver {
 	return &mockCSIDriver{
 		driverInfo: storageframework.DriverInfo{
 			Name:        "csi-mock",
-			FeatureTag:  "",
 			MaxFileSize: storageframework.FileSizeMedium,
 			SupportedFsType: sets.NewString(
 				"", // Default fsType
 			),
 			Capabilities: map[storageframework.Capability]bool{
-				storageframework.CapPersistence:  false,
-				storageframework.CapFsGroup:      false,
-				storageframework.CapExec:         false,
-				storageframework.CapVolumeLimits: true,
+				storageframework.CapPersistence:       false,
+				storageframework.CapFsGroup:           false,
+				storageframework.CapExec:              false,
+				storageframework.CapVolumeLimits:      true,
+				storageframework.CapMultiplePVsSameID: true,
 			},
 		},
-		manifests:              driverManifests,
-		podInfo:                driverOpts.PodInfo,
-		storageCapacity:        driverOpts.StorageCapacity,
-		enableTopology:         driverOpts.EnableTopology,
-		attachable:             !driverOpts.DisableAttach,
-		attachLimit:            driverOpts.AttachLimit,
-		enableNodeExpansion:    driverOpts.EnableNodeExpansion,
-		tokenRequests:          driverOpts.TokenRequests,
-		requiresRepublish:      driverOpts.RequiresRepublish,
-		fsGroupPolicy:          driverOpts.FSGroupPolicy,
-		enableVolumeMountGroup: driverOpts.EnableVolumeMountGroup,
-		embedded:               driverOpts.Embedded,
-		hooks:                  driverOpts.Hooks,
+		manifests:                     driverManifests,
+		podInfo:                       driverOpts.PodInfo,
+		storageCapacity:               driverOpts.StorageCapacity,
+		enableTopology:                driverOpts.EnableTopology,
+		attachable:                    !driverOpts.DisableAttach,
+		attachLimit:                   driverOpts.AttachLimit,
+		enableNodeExpansion:           driverOpts.EnableNodeExpansion,
+		tokenRequests:                 driverOpts.TokenRequests,
+		requiresRepublish:             driverOpts.RequiresRepublish,
+		fsGroupPolicy:                 driverOpts.FSGroupPolicy,
+		enableVolumeMountGroup:        driverOpts.EnableVolumeMountGroup,
+		enableSELinuxMount:            driverOpts.EnableSELinuxMount,
+		enableRecoverExpansionFailure: driverOpts.EnableRecoverExpansionFailure,
+		embedded:                      driverOpts.Embedded,
+		hooks:                         driverOpts.Hooks,
 	}
 }
 
@@ -515,7 +525,7 @@ func (m *mockCSIDriver) GetDriverInfo() *storageframework.DriverInfo {
 func (m *mockCSIDriver) SkipUnsupportedTest(pattern storageframework.TestPattern) {
 }
 
-func (m *mockCSIDriver) GetDynamicProvisionStorageClass(config *storageframework.PerTestConfig, fsType string) *storagev1.StorageClass {
+func (m *mockCSIDriver) GetDynamicProvisionStorageClass(ctx context.Context, config *storageframework.PerTestConfig, fsType string) *storagev1.StorageClass {
 	provisioner := config.GetUniqueDriverName()
 	parameters := map[string]string{}
 	ns := config.Framework.Namespace.Name
@@ -523,18 +533,18 @@ func (m *mockCSIDriver) GetDynamicProvisionStorageClass(config *storageframework
 	return storageframework.GetStorageClass(provisioner, parameters, nil, ns)
 }
 
-func (m *mockCSIDriver) GetSnapshotClass(config *storageframework.PerTestConfig, parameters map[string]string) *unstructured.Unstructured {
+func (m *mockCSIDriver) GetSnapshotClass(ctx context.Context, config *storageframework.PerTestConfig, parameters map[string]string) *unstructured.Unstructured {
 	snapshotter := m.driverInfo.Name + "-" + config.Framework.UniqueName
 	ns := config.Framework.Namespace.Name
 
 	return utils.GenerateSnapshotClassSpec(snapshotter, parameters, ns)
 }
 
-func (m *mockCSIDriver) PrepareTest(f *framework.Framework) (*storageframework.PerTestConfig, func()) {
+func (m *mockCSIDriver) PrepareTest(ctx context.Context, f *framework.Framework) *storageframework.PerTestConfig {
 	m.clientSet = f.ClientSet
 
 	// Create secondary namespace which will be used for creating driver
-	m.driverNamespace = utils.CreateDriverNamespace(f)
+	m.driverNamespace = utils.CreateDriverNamespace(ctx, f)
 	driverns := m.driverNamespace.Name
 	testns := f.Namespace.Name
 
@@ -543,11 +553,11 @@ func (m *mockCSIDriver) PrepareTest(f *framework.Framework) (*storageframework.P
 	} else {
 		ginkgo.By("deploying csi mock driver")
 	}
-	cancelLogging := utils.StartPodLogs(f, m.driverNamespace)
+	cancelLogging := utils.StartPodLogs(ctx, f, m.driverNamespace)
 	cs := f.ClientSet
 
 	// pods should be scheduled on the node
-	node, err := e2enode.GetRandomReadySchedulableNode(cs)
+	node, err := e2enode.GetRandomReadySchedulableNode(ctx, cs)
 	framework.ExpectNoError(err)
 
 	embeddedCleanup := func() {}
@@ -565,6 +575,9 @@ func (m *mockCSIDriver) PrepareTest(f *framework.Framework) (*storageframework.P
 		// this process.
 		podname := "csi-mockplugin-0"
 		containername := "mock"
+
+		// Must keep running even after the test context is cancelled
+		// for cleanup callbacks.
 		ctx, cancel := context.WithCancel(context.Background())
 		serviceConfig := mockservice.Config{
 			DisableAttach:            !m.attachable,
@@ -587,7 +600,6 @@ func (m *mockCSIDriver) PrepareTest(f *framework.Framework) (*storageframework.P
 			Node:       s,
 		}
 		m.embeddedCSIDriver = mockdriver.NewCSIDriver(servers)
-
 		l, err := proxy.Listen(ctx, f.ClientSet, f.ClientConfig(),
 			proxy.Addr{
 				Namespace:     m.driverNamespace.Name,
@@ -596,6 +608,7 @@ func (m *mockCSIDriver) PrepareTest(f *framework.Framework) (*storageframework.P
 				Port:          9000,
 			},
 		)
+
 		framework.ExpectNoError(err, "start connecting to proxy pod")
 		err = m.embeddedCSIDriver.Start(l, m.interceptGRPC)
 		framework.ExpectNoError(err, "start mock driver")
@@ -609,10 +622,10 @@ func (m *mockCSIDriver) PrepareTest(f *framework.Framework) (*storageframework.P
 	} else {
 		// When using the mock driver inside the cluster it has to be reconfigured
 		// via command line parameters.
-		containerArgs = append(containerArgs, "--name=csi-mock-"+f.UniqueName)
+		containerArgs = append(containerArgs, "--drivername=csi-mock-"+f.UniqueName)
 
-		if !m.attachable {
-			containerArgs = append(containerArgs, "--disable-attach")
+		if m.attachable {
+			containerArgs = append(containerArgs, "--enable-attach")
 		}
 
 		if m.enableTopology {
@@ -653,8 +666,14 @@ func (m *mockCSIDriver) PrepareTest(f *framework.Framework) (*storageframework.P
 		TokenRequests:     m.tokenRequests,
 		RequiresRepublish: m.requiresRepublish,
 		FSGroupPolicy:     m.fsGroupPolicy,
+		SELinuxMount:      m.enableSELinuxMount,
+		Features:          map[string][]string{},
 	}
-	cleanup, err := utils.CreateFromManifests(f, m.driverNamespace, func(item interface{}) error {
+
+	if m.enableRecoverExpansionFailure {
+		o.Features["csi-resizer"] = []string{"RecoverVolumeExpansionFailure=true"}
+	}
+	err = utils.CreateFromManifests(ctx, f, m.driverNamespace, func(item interface{}) error {
 		if err := utils.PatchCSIDeployment(config.Framework, o, item); err != nil {
 			return err
 		}
@@ -685,15 +704,14 @@ func (m *mockCSIDriver) PrepareTest(f *framework.Framework) (*storageframework.P
 		"mock",
 		testns,
 		driverns,
-		cleanup,
 		cancelLogging)
 
-	cleanupFunc := func() {
+	ginkgo.DeferCleanup(func(ctx context.Context) {
 		embeddedCleanup()
-		driverCleanupFunc()
-	}
+		driverCleanupFunc(ctx)
+	})
 
-	return config, cleanupFunc
+	return config
 }
 
 func (m *mockCSIDriver) interceptGRPC(ctx context.Context, req interface{}, info *grpc.UnaryServerInfo, handler grpc.UnaryHandler) (resp interface{}, err error) {
@@ -717,7 +735,7 @@ func (m *mockCSIDriver) interceptGRPC(ctx context.Context, req interface{}, info
 	return
 }
 
-func (m *mockCSIDriver) GetCalls() ([]MockCSICall, error) {
+func (m *mockCSIDriver) GetCalls(ctx context.Context) ([]MockCSICall, error) {
 	if m.embedded {
 		return m.calls.Get(), nil
 	}
@@ -732,9 +750,9 @@ func (m *mockCSIDriver) GetCalls() ([]MockCSICall, error) {
 	driverContainerName := "mock"
 
 	// Load logs of driver pod
-	log, err := e2epod.GetPodLogs(m.clientSet, m.driverNamespace.Name, driverPodName, driverContainerName)
+	log, err := e2epod.GetPodLogs(ctx, m.clientSet, m.driverNamespace.Name, driverPodName, driverContainerName)
 	if err != nil {
-		return nil, fmt.Errorf("could not load CSI driver logs: %s", err)
+		return nil, fmt.Errorf("could not load CSI driver logs: %w", err)
 	}
 
 	logLines := strings.Split(log, "\n")
@@ -777,7 +795,7 @@ func InitGcePDCSIDriver() storageframework.TestDriver {
 	return &gcePDCSIDriver{
 		driverInfo: storageframework.DriverInfo{
 			Name:        GCEPDCSIDriverName,
-			FeatureTag:  "[Serial]",
+			TestTags:    []interface{}{framework.WithSerial()},
 			MaxFileSize: storageframework.FileSizeMedium,
 			SupportedSizeRange: e2evolume.SizeRange{
 				Min: "5Gi",
@@ -798,12 +816,16 @@ func InitGcePDCSIDriver() storageframework.TestDriver {
 				storageframework.CapMultiPODs:   true,
 				// GCE supports volume limits, but the test creates large
 				// number of volumes and times out test suites.
-				storageframework.CapVolumeLimits:        false,
-				storageframework.CapTopology:            true,
-				storageframework.CapControllerExpansion: true,
-				storageframework.CapOnlineExpansion:     true,
-				storageframework.CapNodeExpansion:       true,
-				storageframework.CapSnapshotDataSource:  true,
+				storageframework.CapVolumeLimits:                   false,
+				storageframework.CapTopology:                       true,
+				storageframework.CapControllerExpansion:            true,
+				storageframework.CapOfflineExpansion:               true,
+				storageframework.CapOnlineExpansion:                true,
+				storageframework.CapNodeExpansion:                  true,
+				storageframework.CapSnapshotDataSource:             true,
+				storageframework.CapReadWriteOncePod:               true,
+				storageframework.CapMultiplePVsSameID:              true,
+				storageframework.CapFSResizeFromSourceNotSupported: true, //TODO: remove when CI tests use the fixed driver with: https://github.com/kubernetes-sigs/gcp-compute-persistent-disk-csi-driver/pull/972
 			},
 			RequiredAccessModes: []v1.PersistentVolumeAccessMode{v1.ReadWriteOnce},
 			TopologyKeys:        []string{GCEPDCSIZoneTopologyKey},
@@ -832,12 +854,14 @@ func (g *gcePDCSIDriver) SkipUnsupportedTest(pattern storageframework.TestPatter
 	if pattern.FsType == "xfs" {
 		e2eskipper.SkipUnlessNodeOSDistroIs("ubuntu", "custom")
 	}
-	if pattern.FeatureTag == "[Feature:Windows]" {
-		e2eskipper.Skipf("Skipping tests for windows since CSI does not support it yet")
+	for _, tag := range pattern.TestTags {
+		if framework.TagsEqual(tag, feature.Windows) {
+			e2eskipper.Skipf("Skipping tests for windows since CSI does not support it yet")
+		}
 	}
 }
 
-func (g *gcePDCSIDriver) GetDynamicProvisionStorageClass(config *storageframework.PerTestConfig, fsType string) *storagev1.StorageClass {
+func (g *gcePDCSIDriver) GetDynamicProvisionStorageClass(ctx context.Context, config *storageframework.PerTestConfig, fsType string) *storagev1.StorageClass {
 	ns := config.Framework.Namespace.Name
 	provisioner := g.driverInfo.Name
 
@@ -850,14 +874,14 @@ func (g *gcePDCSIDriver) GetDynamicProvisionStorageClass(config *storageframewor
 	return storageframework.GetStorageClass(provisioner, parameters, &delayedBinding, ns)
 }
 
-func (g *gcePDCSIDriver) GetSnapshotClass(config *storageframework.PerTestConfig, parameters map[string]string) *unstructured.Unstructured {
+func (g *gcePDCSIDriver) GetSnapshotClass(ctx context.Context, config *storageframework.PerTestConfig, parameters map[string]string) *unstructured.Unstructured {
 	snapshotter := g.driverInfo.Name
 	ns := config.Framework.Namespace.Name
 
 	return utils.GenerateSnapshotClassSpec(snapshotter, parameters, ns)
 }
 
-func (g *gcePDCSIDriver) PrepareTest(f *framework.Framework) (*storageframework.PerTestConfig, func()) {
+func (g *gcePDCSIDriver) PrepareTest(ctx context.Context, f *framework.Framework) *storageframework.PerTestConfig {
 	testns := f.Namespace.Name
 	cfg := &storageframework.PerTestConfig{
 		Driver:    g,
@@ -867,15 +891,15 @@ func (g *gcePDCSIDriver) PrepareTest(f *framework.Framework) (*storageframework.
 
 	if framework.ProviderIs("gke") {
 		framework.Logf("The csi gce-pd driver is automatically installed in GKE. Skipping driver installation.")
-		return cfg, func() {}
+		return cfg
 	}
 
 	ginkgo.By("deploying csi gce-pd driver")
 	// Create secondary namespace which will be used for creating driver
-	driverNamespace := utils.CreateDriverNamespace(f)
+	driverNamespace := utils.CreateDriverNamespace(ctx, f)
 	driverns := driverNamespace.Name
 
-	cancelLogging := utils.StartPodLogs(f, driverNamespace)
+	cancelLogging := utils.StartPodLogs(ctx, f, driverNamespace)
 	// It would be safer to rename the gcePD driver, but that
 	// hasn't been done before either and attempts to do so now led to
 	// errors during driver registration, therefore it is disabled
@@ -898,12 +922,12 @@ func (g *gcePDCSIDriver) PrepareTest(f *framework.Framework) (*storageframework.
 		"test/e2e/testing-manifests/storage-csi/gce-pd/controller_ss.yaml",
 	}
 
-	cleanup, err := utils.CreateFromManifests(f, driverNamespace, nil, manifests...)
+	err := utils.CreateFromManifests(ctx, f, driverNamespace, nil, manifests...)
 	if err != nil {
 		framework.Failf("deploying csi gce-pd driver: %v", err)
 	}
 
-	if err = WaitForCSIDriverRegistrationOnAllNodes(GCEPDCSIDriverName, f.ClientSet); err != nil {
+	if err = WaitForCSIDriverRegistrationOnAllNodes(ctx, GCEPDCSIDriverName, f.ClientSet); err != nil {
 		framework.Failf("waiting for csi driver node registration on: %v", err)
 	}
 
@@ -912,26 +936,26 @@ func (g *gcePDCSIDriver) PrepareTest(f *framework.Framework) (*storageframework.
 		"gce-pd",
 		testns,
 		driverns,
-		cleanup,
 		cancelLogging)
+	ginkgo.DeferCleanup(cleanupFunc)
 
 	return &storageframework.PerTestConfig{
 		Driver:          g,
 		Prefix:          "gcepd",
 		Framework:       f,
 		DriverNamespace: driverNamespace,
-	}, cleanupFunc
+	}
 }
 
 // WaitForCSIDriverRegistrationOnAllNodes waits for the CSINode object to be updated
 // with the given driver on all schedulable nodes.
-func WaitForCSIDriverRegistrationOnAllNodes(driverName string, cs clientset.Interface) error {
-	nodes, err := e2enode.GetReadySchedulableNodes(cs)
+func WaitForCSIDriverRegistrationOnAllNodes(ctx context.Context, driverName string, cs clientset.Interface) error {
+	nodes, err := e2enode.GetReadySchedulableNodes(ctx, cs)
 	if err != nil {
 		return err
 	}
 	for _, node := range nodes.Items {
-		if err := WaitForCSIDriverRegistrationOnNode(node.Name, driverName, cs); err != nil {
+		if err := WaitForCSIDriverRegistrationOnNode(ctx, node.Name, driverName, cs); err != nil {
 			return err
 		}
 	}
@@ -939,7 +963,7 @@ func WaitForCSIDriverRegistrationOnAllNodes(driverName string, cs clientset.Inte
 }
 
 // WaitForCSIDriverRegistrationOnNode waits for the CSINode object generated by the node-registrar on a certain node
-func WaitForCSIDriverRegistrationOnNode(nodeName string, driverName string, cs clientset.Interface) error {
+func WaitForCSIDriverRegistrationOnNode(ctx context.Context, nodeName string, driverName string, cs clientset.Interface) error {
 	framework.Logf("waiting for CSIDriver %v to register on node %v", driverName, nodeName)
 
 	// About 8.6 minutes timeout
@@ -950,7 +974,7 @@ func WaitForCSIDriverRegistrationOnNode(nodeName string, driverName string, cs c
 	}
 
 	waitErr := wait.ExponentialBackoff(backoff, func() (bool, error) {
-		csiNode, err := cs.StorageV1().CSINodes().Get(context.TODO(), nodeName, metav1.GetOptions{})
+		csiNode, err := cs.StorageV1().CSINodes().Get(ctx, nodeName, metav1.GetOptions{})
 		if err != nil && !apierrors.IsNotFound(err) {
 			return false, err
 		}
@@ -984,31 +1008,22 @@ func tryFunc(f func()) error {
 func generateDriverCleanupFunc(
 	f *framework.Framework,
 	driverName, testns, driverns string,
-	driverCleanup, cancelLogging func()) func() {
-
-	cleanupHandle := new(framework.CleanupActionHandle)
+	cancelLogging func()) func(ctx context.Context) {
 
 	// Cleanup CSI driver and namespaces. This function needs to be idempotent and can be
 	// concurrently called from defer (or AfterEach) and AfterSuite action hooks.
-	cleanupFunc := func() {
+	cleanupFunc := func(ctx context.Context) {
 		ginkgo.By(fmt.Sprintf("deleting the test namespace: %s", testns))
 		// Delete the primary namespace but it's okay to fail here because this namespace will
 		// also be deleted by framework.Aftereach hook
-		tryFunc(func() { f.DeleteNamespace(testns) })
+		_ = tryFunc(func() { f.DeleteNamespace(ctx, testns) })
 
 		ginkgo.By(fmt.Sprintf("uninstalling csi %s driver", driverName))
-		tryFunc(driverCleanup)
-		tryFunc(cancelLogging)
+		_ = tryFunc(cancelLogging)
 
 		ginkgo.By(fmt.Sprintf("deleting the driver namespace: %s", driverns))
-		tryFunc(func() { f.DeleteNamespace(driverns) })
-		// cleanup function has already ran and hence we don't need to run it again.
-		// We do this as very last action because in-case defer(or AfterEach) races
-		// with AfterSuite and test routine gets killed then this block still
-		// runs in AfterSuite
-		framework.RemoveCleanupAction(*cleanupHandle)
+		_ = tryFunc(func() { f.DeleteNamespace(ctx, driverns) })
 	}
 
-	*cleanupHandle = framework.AddCleanupAction(cleanupFunc)
 	return cleanupFunc
 }
